package com.gic.store.service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.store.dto.ClerkTmpDTO;

import java.util.List;
import java.util.Map;

/**
 * @ClassName: ClerkImportApiService
 * @Description: 导购导入service
 * @author zhiwj
 * @date 2019/7/31 11:37
 */
public interface ClerkImportApiService {
    /**
     * @Title: hasImportingClerk
     * @Description: 企业下是否存在 正在导入导购的任务
     * @author zhiwj 
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Boolean>
     * @throws 
     */
    ServiceResponse<Boolean> hasImportingClerk(Integer enterpriseId);

    /**
     * @Title: delClerkTmp
     * @Description: 删除企业下所有导购
     * @author zhiwj 
     * @param enterpriseId
     * @return void
     * @throws 
     */
    ServiceResponse<Void> delClerkTmp(Integer enterpriseId);
    
    /**
     * @Title: save
     * @Description: 保存clerkTmp
     * @author zhiwj 
     * @param bean
     * @return void
     * @throws 
     */
    ServiceResponse<Void> save(ClerkTmpDTO bean);

    /**
     * @Title: importDataToClerk
     * @Description: 导入导购
     * @author zhiwj
     * @param enterpriseId 企业id
     * @param userId 操作人
     * @param uuId 上传任务的唯一标识
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.String>
     * @throws
     */
    ServiceResponse<String> importDataToClerk(Integer enterpriseId, String userId, String uuId);

    /**
     * @Title: listClerkTmp
     * @Description: 分页查导购导入日志
     * @author zhiwj 
     * @param enterpriseId 企业id
     * @param isSuccess true:查询所有成功的  false:查询所有失败的
     * @param currentPage
     * @param pageSize
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page>
     * @throws 
     */
    ServiceResponse<Page> listClerkTmp(Integer enterpriseId, boolean isSuccess, Integer currentPage, Integer pageSize);
    
    /**
     * @Title: getClerkBatchProcess
     * @Description: 查询导入进度
     * @author zhiwj 
     * @param enterpriseId 企业id
     * @param uuId 导入任务的唯一标识
     * @param currentPage
     * @param pageSize
     * @return com.gic.api.base.commons.ServiceResponse<java.util.Map<java.lang.String,java.lang.Object>>
     * @throws 
     */
    ServiceResponse<Map<String, Object>> getClerkBatchProcess(Integer enterpriseId, String uuId, Integer currentPage, Integer pageSize);

    /**
     * @Title: listClerkTmp
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param isRight true:查询所有成功的  false:查询所有失败的
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.store.dto.ClerkTmpDTO>>
     * @throws
     */
    ServiceResponse<List<ClerkTmpDTO>> listClerkTmp(Integer enterpriseId, Boolean isRight);
    
    /**
     * @Title: pageError
     * @Description: 查询错误日志
     * @author zhiwj 
     * @param enterpriseId 企业id
     * @param currentPage
     * @param pageSize
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page>
     * @throws 
     */
    ServiceResponse<Page> pageError(Integer enterpriseId, Integer currentPage, Integer pageSize);
    
    /**
     * @Title: listAllClerkTmp
     * @Description: 查询所有
     * @author zhiwj 
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.store.dto.ClerkTmpDTO>>
     * @throws 
     */
    ServiceResponse<List<ClerkTmpDTO>> listAllClerkTmp(Integer enterpriseId);
    
    /**
     * @Title: listError
     * @Description: 查询所有错误日志
     * @author zhiwj 
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<java.util.List<java.lang.String>>>
     * @throws
     */
    ServiceResponse<List<List<String>>> listError(Integer enterpriseId);
}
