package com.gic.store.service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.store.dto.CityDTO;
import com.gic.store.dto.CountyDTO;
import com.gic.store.dto.ProvinceDTO;

import javax.xml.ws.Service;
import java.util.List;


public interface ProvincesApiService {
   
   /** @Description: 查询所有省份
   * @author taogs
    * @Date 13:57 2019/8/5
    * @Param 
    * @return 
   */
    ServiceResponse<List<ProvinceDTO>> selectAllProvince();

    
    /** @Description: 查询所有城市
     * @author taogs
     * @Date 14:07 2019/8/5
     * @Param 
     * @return 
    */
    ServiceResponse<List<CityDTO>> selectAllCity();
    
    /** @Description: 通过城市id查询城市
     * @author taogs
     * @Date 11:10 2019/12/9
     * @Param 
     * @return 
    */
    ServiceResponse<CityDTO> getCityByCityId(String cityId);

    /** @Description: 通过省id查询城市
     * @author taogs
     * @Date 14:29 2019/12/4
     * @Param 
     * @return 
    */
    ServiceResponse<List<CityDTO>> selectCityByProvinceId(String provinceId);

  /** @Description: 查询所有地区
     * @author taogs
     * @Date 14:07 2019/8/5
     * @Param 
     * @return 
    */
    ServiceResponse<List<CountyDTO>> selectAllCounty();

    
    /** @Description: 通过地区id查询
     * @author taogs
     * @Date 11:11 2019/12/9
     * @Param 
     * @return 
    */
    ServiceResponse<CountyDTO> getCountyByCountyId(String countyId);
    
    /** @Description: 通过城市id查询地区
     * @author taogs
     * @Date 14:38 2019/12/4
     * @Param 
     * @return 
    */
    ServiceResponse<List<CountyDTO>> selectCountyByCityId(String cityId);

    
    /** @Description: 通过城市id查询地区id
     * @author taogs
     * @Date 14:44 2019/12/4
     * @Param 
     * @return 
    */
    ServiceResponse<List<String>> selectCountyIdByCityId(String cityId);
    
   /** @Description: 根据省份名称查询code
    * @author taogs
    * @Date 13:59 2019/8/5
    * @Param 
    * @return 
   */
    ServiceResponse<List<ProvinceDTO>> queryProvinces(String search);

    
    /** @Description: 根据城市名称查询code
     * @author taogs
     * @Date 14:08 2019/8/5
     * @Param 
     * @return 
    */
    ServiceResponse<List<CityDTO>> queryCity(String search);

    
    /** @Description: 根据地区名称查询地区code
     * @author taogs
     * @Date 14:08 2019/8/5
     * @Param 
     * @return 
    */
    ServiceResponse<List<CountyDTO>> queryCounty(String search);

    ServiceResponse<ProvinceDTO> getProvinceById(String provinceId);

    ServiceResponse<String> getProvinceNameByAreaId(String aredId);
    
    /** @Description: 热门城市
     * @author taogs
     * @Date 17:35 2019/12/4
     * @Param 
     * @return 
    */
    ServiceResponse<List<CityDTO>> listHotCity(Integer enterpriseId, Integer brandId);

    ServiceResponse<List<CityDTO>> listEnterpriseCity(Integer enterpriseId);
}
