package com.gic.store.service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.store.dto.open.ClerkDTO;
import com.gic.store.dto.open.StoreOpenDTO;

/**
 * 门店、导购对外API接口
 * @ClassName: StoreOpenApiService 
 * @Description:  
 * @author guojuxing 
 * @date 2020/2/25 3:48 PM 
 */
public interface StoreOpenApiService {

    /**
     * 获取gic端和对外API之间门店自定义字段的匹配数据
     * @Title: getApiFieldJson 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 商户ID
     * @param regionId 门店域ID
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String> 
     */
    ServiceResponse<String> getApiFieldJson(Integer enterpriseId, Integer regionId);

    /**
     * 查询门店详情
     * @Title: getStoreDetail 
     * @Description:
     * @author guojuxing
     * @param appId
     * @param enterpriseId 商户ID
     * @param storeCode 门店代码
     * @param regionId 门店域ID
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String> 
     */
    ServiceResponse<String> getStoreDetail(String appId, Integer enterpriseId, String storeCode, Integer regionId);

    /**
     * 门店新增接口
     * @Title: addStore
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeOpenDTO
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.String>
     * @throws
     */
    ServiceResponse<String> addStore(String appId, Integer enterpriseId, StoreOpenDTO storeOpenDTO);


    /**
     * 门店修改接口
     * @Title: editStore
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeOpenDTO
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.String>
     * @throws
     */
    ServiceResponse<String> editStore(String appId, Integer enterpriseId, StoreOpenDTO storeOpenDTO);

    /**
     * 门店列表接口
     * @Title: pageStore
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param regionId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.String>
     * @throws
     */
    ServiceResponse<String> pageStore(String appId, Integer enterpriseId, Integer regionId, Integer pageNum, Integer pageSize);


    /**
     * 查询导购详情
     * @Title: getClerkDetail 
     * @Description:
     * @author guojuxing
     * @param appId
     * @param enterpriseId 商户ID
     * @param storeCode 门店code
     * @param clerkCode 导购code
     * @param regionId 门店域ID
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String> 
     */
    ServiceResponse<String> getClerkDetail(String appId, Integer enterpriseId, String storeCode, String clerkCode, Integer regionId);

    ServiceResponse<Void> addClerk(String appId, Integer enterpriseId, ClerkDTO clerkInfo);

    ServiceResponse<Void> editClerk(String appId, Integer enterpriseId, ClerkDTO clerkInfo);

    /**
     * 分页查询导购列表
     * @Title: pageClerk 
     * @Description:
     * @author guojuxing
     * @param appId 应用ID
     * @param enterpriseId 商户ID
     * @param storeCode 门店code 如果为空，则查询所有域下的
     * @param regionId 门店域ID
     * @param pageNum 第几页
     * @param pageSize 每页多少条
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String> 
     */
    ServiceResponse<String> pageClerk(String appId, Integer enterpriseId, String storeCode, Integer regionId, Integer pageNum, Integer pageSize);
}
