package com.gic.store.dao.mapper;

import com.gic.store.entity.TabStoreFieldSelect;
import org.apache.ibatis.annotations.Param;

import java.util.List;

public interface TabStoreFieldSelectMapper {
    /**
     * 根据主键删除
     *
     * @param storeFieldSelectId 主键
     * @return 更新条目数
     */
    int deleteByPrimaryKey(Integer storeFieldSelectId);

    int deleteBatch(@Param("storeFieldSelectIdList") List<Integer> storeFieldSelectIdList);

    /**
     * 插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insert(TabStoreFieldSelect record);

    /**
     * 动态插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insertSelective(TabStoreFieldSelect record);

    /**
     * 根据主键查询
     *
     * @param storeFieldSelectId 主键
     * @return 实体对象
     */
    TabStoreFieldSelect selectByPrimaryKey(Integer storeFieldSelectId);

    /**
     * 根据主键动态更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKeySelective(TabStoreFieldSelect record);

    /**
     * 根据主键更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKey(TabStoreFieldSelect record);

    /**
     * 查询下一个排序属性值
     * @param storeFieldId
     * @param sort
     * @return
     */
    TabStoreFieldSelect selectNextByStoreFieldIdAndSort(@Param("storeFieldId") Integer storeFieldId, @Param("sort") Double sort);

    /**
     * 查询上一个排序属性值
     * @param storeFieldId
     * @param sort
     * @return
     */
    TabStoreFieldSelect selectPrevByStoreFieldIdAndSort(@Param("storeFieldId") Integer storeFieldId, @Param("sort") Double sort);

    /**
     * 查询某字段最小排序值
     * @param storeFieldId
     * @return
     */
    Double getMinSort(@Param("storeFieldId") Integer storeFieldId);

    /**
     * 查询某字段最大排序值
     * @param storeFieldId
     * @return
     */
    Double getMaxSort(@Param("storeFieldId") Integer storeFieldId);

    /**
     * 查询几条连续的记录
     * @param storeFieldId
     * @param start 开始的位置
     * @param size  查询几条条
     * @param storeFieldSelectId 要除外的id
     * @return
     */
    List<TabStoreFieldSelect> listTwoElementExceptId(@Param("storeFieldId") Integer storeFieldId, @Param("start") int start,
                                                     @Param("size") int size, @Param("storeFieldSelectId") Integer storeFieldSelectId);

    /**
     * 分页查询
     * @param storeFieldId
     * @param search
     * @return
     */
    List<TabStoreFieldSelect> listStoreFieldSelect(@Param("storeFieldId") Integer storeFieldId, @Param("search") String search);

    int isRepeatCode(@Param("storeFieldId") Integer storeFieldId, @Param("storeFieldSelectId") Integer storeFieldSelectId,
                     @Param("storeFieldSelectCode") String storeFieldSelectCode);

    int isRepeatName(@Param("storeFieldId") Integer storeFieldId, @Param("storeFieldSelectId") Integer storeFieldSelectId,
                     @Param("storeFieldSelectName") String storeFieldSelectName);
    List<TabStoreFieldSelect> listStoreFieldSelectByIds(@Param("storeFileIdSelectIdList") List<Integer> storeFileIdSelectIdList);

}