package com.gic.store.dao.mapper;

import com.gic.store.dto.StoreRegionDTO;
import com.gic.store.entity.TabStoreRegion;
import org.apache.ibatis.annotations.Param;

import java.util.List;

public interface TabStoreRegionMapper {
    /**
     * 根据主键删除
     *
     * @param regionId 主键
     * @return 更新条目数
     */
    int deleteByPrimaryKey(Integer regionId);

    /**
     * 插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insert(TabStoreRegion record);

    /**
     * 动态插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insertSelective(TabStoreRegion record);

    /**
     * 根据主键查询
     * @param enterpriseId 商户id
     * @param regionId 主键
     * @return 实体对象
     */
    TabStoreRegion selectByPrimaryKey(@Param("enterpriseId") Integer enterpriseId, @Param("regionId") Integer regionId);

    /**
     * 根据主键动态更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKeySelective(TabStoreRegion record);

    /**
     * 根据主键更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKey(TabStoreRegion record);

    /**
     * 查询门店域列表
     * @param enterpriseId 企业id
     * @param search        域名/code
     * @return
     */
    List<StoreRegionDTO> listStoreRegion(@Param("enterpriseId") Integer enterpriseId, @Param("search") String search, @Param("ownType") Integer ownType);

    /**
     * 查询符合条件的个数； 除id所有值取等于计算， id取不等于计算
     * @param storeRegion
     * @return
     */
    int countBySelective(StoreRegionDTO storeRegion);

    List<StoreRegionDTO> listStoreRegionByIds(@Param("enterpriseId") Integer enterpriseId, @Param("storeRegionIdList") List<Integer> storeRegionIdList, @Param("ownType") Integer ownType);

    StoreRegionDTO getStoreRegionByRegionName(@Param("enterpriseId") Integer enterpriseId, @Param("regionName") String regionName);
}