package com.gic.store.service;

import com.gic.store.dto.ClerkTmpDTO;
import com.gic.store.entity.TabClerkTmp;
import com.github.pagehelper.Page;

import java.util.List;

/**
 * @ClassName: ClerkImportService
 * @Description:
 * @author zhiwj
 * @date 2019/7/31 11:17
 */
public interface ClerkImportService {
    /**
     * @Title: hasImportingClerk
     * @Description: 查询企业下是否有 正在导入导购的任务
     * @author zhiwj
     * @param enterpriseId 企业id
     * @return java.lang.Boolean
     * @throws
     */
    Boolean hasImportingClerk(Integer enterpriseId);

    /**
     * @Title: delClerkTmp
     * @Description: 删除企业下所有的导入导购tmp
     * @author zhiwj
     * @param enterpriseId 企业id
     * @return void
     * @throws
     */
    void delClerkTmp(Integer enterpriseId);

    /**
     * @Title: saveByOwner
     * @Description: 保存一个tmp实体
     * @author zhiwj
     * @param bean bean
     * @return void
     * @throws
     */
    void save(ClerkTmpDTO bean);

    /**
     * @Title: listClerkTmp
     * @Description: 查询导入的列表
     * @author zhiwj
     * @param enterpriseId 企业id
     * @param isSuccess true:查询所有成功的  false:查询所有失败的
     * @param currentPage 页码
     * @param pageSize 页大小
     * @return com.github.pagehelper.Page
     * @throws
     */
    Page listClerkTmp(Integer enterpriseId, boolean isSuccess, Integer currentPage, Integer pageSize);

    /**
     * @Title: listUnImportClerk
     * @Description: 查询所有等待导入的导购
     * @author zhiwj
     * @param enterpriseId 企业id
     * @return java.util.List<com.gic.store.entity.TabClerkTmp>
     * @throws
     */
    List<TabClerkTmp> listUnImportClerk(String enterpriseId);

    /**
     * @Title: updateData
     * @Description: 更新导入导购tmp的状态
     * @author zhiwj
     * @param t 导购tmp
     * @return void
     * @throws
     */
    void updateData(ClerkTmpDTO t);

    /**
     * @Title: countByTaskTotal
     * @Description: 导入的总任务数量
     * @author zhiwj
     * @param enterpriseId 企业id
     * @param uuId 每次导入的唯一参数
     * @return java.lang.Integer
     * @throws
     */
    Integer countByTaskTotal(Integer enterpriseId, String uuId);
    
    /**
     * @Title: countByTaskExecNum
     * @Description: 导入的已执行任务数量
     * @author zhiwj 
     * @param enterpriseId 企业id
     * @param uuId 每次导入的唯一参数
     * @return java.lang.Integer
     * @throws 
     */
    Integer countByTaskExecNum(Integer enterpriseId, String uuId);

    /**
     * @Title: countByTaskFailNum
     * @Description: 导入的失败任务数量
     * @author zhiwj 
     * @param enterpriseId
     * @param uuId
     * @return java.lang.Integer
     * @throws 
     */
    Integer countByTaskFailNum(Integer enterpriseId, String uuId);

    /**
     * @Title: listClerkTmp
     * @Description: 查询导入的列表
     * @author zhiwj
     * @param enterpriseId 企业id
     * @param isSuccess true:查询所有成功的  false:查询所有失败的
     * @return java.util.List<com.gic.store.entity.TabClerkTmp>
     * @throws
     */
    List<TabClerkTmp> listClerkTmp(Integer enterpriseId, Boolean isSuccess);

    /**
     * @Title: listAllClerkTmp
     * @Description: 查询导入的列表
     * @author zhiwj 
     * @param enterpriseId 企业id
     * @return java.util.List<com.gic.store.entity.TabClerkTmp>
     * @throws 
     */
    List<TabClerkTmp> listAllClerkTmp(Integer enterpriseId);
}
