package com.gic.store.service.impl;

import com.alibaba.fastjson.JSON;
import com.gic.commons.util.EntityUtil;
import com.gic.store.dao.mapper.TabStoreExtendMapper;
import com.gic.store.dto.StoreExtendDTO;
import com.gic.store.entity.TabStoreExtend;
import com.gic.store.entity.TabStoreField;
import com.gic.store.service.StoreExtendService;
import com.gic.store.service.StoreFieldService;
import com.gic.store.utils.field.FieldBase;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.Date;
import java.util.List;

/**
 * @author zhiwj
 * @date 2019/7/3
 */
@Service
public class StoreExtendServiceImpl implements StoreExtendService {

    @Autowired
    private TabStoreExtendMapper tabStoreExtendMapper;

    @Autowired
    private StoreFieldService storeFieldService;

    @Override
    public String saveOrUpdate(Integer enterpriseId, Integer storeInfoId, List<StoreExtendDTO> storeExtendList) {
        if (CollectionUtils.isNotEmpty(storeExtendList)) {

            this.tabStoreExtendMapper.delete(enterpriseId, storeInfoId);

            List<TabStoreExtend> tabStoreExtendList = EntityUtil.changeEntityListByOrika(TabStoreExtend.class, storeExtendList);
            for (TabStoreExtend tabStoreExtend : tabStoreExtendList) {
                TabStoreField field = storeFieldService.getStoreFieldById(tabStoreExtend.getStoreFieldId());
                String fieldDetail = field.getStoreFieldDetail();
                FieldBase fieldBase = JSON.parseObject(fieldDetail, FieldBase.class);
                tabStoreExtend.setEnterpriseId(enterpriseId);
                tabStoreExtend.setStoreInfoId(storeInfoId);
                tabStoreExtend.setCreateTime(new Date());
                if (!fieldBase.getAllowRepeat()) {
                    // 不允许重复
                    if (StringUtils.isNotBlank(tabStoreExtend.getValue())) {
//                        int count = tabStoreExtendMapper.countExtendByValue(field.getStoreFieldId(), tabStoreExtend.getValue(), tabStoreExtend.getStoreExtendId());
//                        if (count > 0) {
//                            return field.getStoreFieldName() + "不允许重复";
//                        }
                        tabStoreExtendMapper.insertNotRepeat(tabStoreExtend);
                    }
                } else {
                    tabStoreExtendMapper.insertSelective(tabStoreExtend);
                }
            }
        }
        return null;
    }

    @Override
    public List<TabStoreExtend> listByStoreInfoId(Integer enterpriseId, Integer storeInfoId) {
        return tabStoreExtendMapper.list(enterpriseId, storeInfoId);
    }

    @Override
    public List<StoreExtendDTO> convertStoreExtendToDTO(List<TabStoreExtend> extendList) {
        if (CollectionUtils.isNotEmpty(extendList)) {
            List<StoreExtendDTO> extendDTOList = EntityUtil.changeEntityListByOrika(StoreExtendDTO.class, extendList);
            for (StoreExtendDTO storeExtendDTO : extendDTOList) {
                TabStoreField field = storeFieldService.getStoreFieldById(storeExtendDTO.getStoreFieldId());
                if (field != null) {
                    storeExtendDTO.setStoreFieldName(field.getStoreFieldName());
                    storeExtendDTO.setStoreFieldCode(field.getStoreFieldCode());
                    storeExtendDTO.setStoreFieldType(field.getStoreFieldType());
                    storeExtendDTO.setStoreFieldDetail(field.getStoreFieldDetail());
                    storeExtendDTO.setErpRelCode(field.getErpRelCode());
                }
            }
            return extendDTOList;
        } else {
            return Collections.emptyList();
        }
    }

    @Override
    public TabStoreExtend getStoreExtendByStoreInfoIdAndFieldId(Integer storeInfoId, Integer storeFieldId) {
        return tabStoreExtendMapper.getStoreExtendByStoreInfoIdAndStoreFieldId(storeInfoId, storeFieldId);
    }

    @Override
    public boolean storeExtendHasStoreField(Integer storeField) {
        return tabStoreExtendMapper.countStoreExtendStoreField(storeField, null) > 0;
    }

    @Override
    public boolean storeExtendHasStoreFieldSelect(Integer storeField, Integer storeFieldSelectId) {
        return tabStoreExtendMapper.countStoreExtendStoreField(storeField, storeFieldSelectId) > 0;
    }
}