package com.gic.store.service.outer.impl;

import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.CollectionUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.enterprise.constants.Constants;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.mq.sdk.GicMQClient;
import com.gic.store.constant.ClerkLogReasonEnum;
import com.gic.store.constant.ImportClerkBatchResultEnum;
import com.gic.store.constant.StoreLogTypeEnum;
import com.gic.store.constant.StoreOwnTypeEnum;
import com.gic.store.dto.*;
import com.gic.store.dto.clerk.OrderAddClerkDTO;
import com.gic.store.dto.clerk.PosAddClerkDTO;
import com.gic.store.dto.clerk.ProcessBatchClerkDTO;
import com.gic.store.entity.TabClerk;
import com.gic.store.entity.TabStoreGroup;
import com.gic.store.entity.TabStoreRegion;
import com.gic.store.service.*;
import com.gic.weimob.api.dto.WeimobGuiderSynDTO;
import com.gic.weimob.api.service.WeimobStoreSiteService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author zhiwj
 * @date 2019/7/16
 */
@Service("clerkApiService")
public class ClerkApiServiceImpl implements ClerkApiService {

    private static final Logger logger = LogManager.getLogger(ClerkApiServiceImpl.class);

    @Autowired
    private ClerkService clerkService;
    @Autowired
    private StoreGroupService storeGroupService;
    @Autowired
    private StoreService storeService;
    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private ClerkLogService clerkLogService;
    @Autowired
    private StoreLogService storeLogService;
    @Autowired
    private StoreRegionService storeRegionService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private WeimobStoreSiteService weimobStoreSiteService;

    @Override
    public ServiceResponse saveOrUpdate(ClerkDTO clerkDTO) {
        if (validStoreIsOther(clerkDTO.getEnterpriseId(), clerkDTO.getStoreInfoId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "权限不足");
        }
        // 校验参数
        if (clerkService.isRepeatByClerkCode(clerkDTO.getStoreInfoId(), clerkDTO.getClerkCode(),
                clerkDTO.getClerkId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店下导购代码重复");
        }
        if (StringUtils.isNotBlank(clerkDTO.getPhoneNumber())) {
            String nationcode = clerkDTO.getNationcode();
            if (StringUtils.isBlank(nationcode)) {
                nationcode = Constants.NATION_CODE;
            }
            //默认加上+
            if (!nationcode.contains("+")) {
                nationcode = "+" + nationcode;
            }
            clerkDTO.setNationcode(nationcode);

            if (clerkService.isRepeatByPhoneNumber(clerkDTO.getStoreInfoId(), clerkDTO.getPhoneNumber(),
                    clerkDTO.getClerkId(), clerkDTO.getNationcode())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店下导购手机号重复");
            }
        }
        if (clerkService.isRepeatByClerkName(clerkDTO.getStoreInfoId(), clerkDTO.getClerkName(),
                clerkDTO.getClerkId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店下导购名称重复");
        }
        // 店长逻辑
        // 如果门店下已经存在有导购是店长， 并且当前导购是店长 则做替换
        // 如果门店下没有导购是店长，将当前导购设置成店长
        TabClerk clerkLeader = clerkService.getClerkLeaderByStoreInfoId(clerkDTO.getEnterpriseId(), clerkDTO.getStoreInfoId());
        if (clerkLeader != null && Integer.valueOf(1).equals(clerkDTO.getClerkType())) {
            clerkLeader.setClerkType(0);
            clerkService.update(clerkLeader);
            clerkDTO.setClerkType(1);
        } else {
            clerkDTO.setClerkType(1);
        }

        if (clerkDTO.getClerkId() == null) {
            //            Integer saveClerkLogBySaveClerk(Integer reason, Integer enterpriseId, Integer clerkId, String remark, Integer operatorId, String operatorName);
            Integer id = clerkService.save(clerkDTO);
            clerkDTO.setClerkId(id);
            clerkLogService.saveClerkLogBySaveClerk(clerkDTO.getEnterpriseId(), id, clerkDTO.getReason(),
                    clerkDTO.getRemark(), clerkDTO.getOperatorId(), clerkDTO.getOperatorName());
            storeLogByAddClerk(clerkDTO);
        } else {
            TabClerk oldClerk = this.clerkService.getById(clerkDTO.getClerkId());
            clerkService.update(clerkDTO);
            clerkLogService.saveClerkLogByUpdateClerk(oldClerk, clerkDTO, clerkDTO.getEnterpriseId(),
                    clerkDTO.getClerkId(), clerkDTO.getReason(), clerkDTO.getRemark(), clerkDTO.getOperatorId(),
                    clerkDTO.getOperatorName());
        }
        return ServiceResponse.success(clerkDTO.getClerkId());
    }

    @Override
    public ServiceResponse<Integer> saveClerkForOrder(String clerkName, String clerkCode, Integer storeInfoId,
            Integer enterpriseId, String orderNum) {
//        ServiceResponse<StoreDTO> storeResponse = storeApiService.getStoreById(enterpriseId, storeInfoId);
//        if (storeResponse.getResult() != null) {
//            StoreDTO store = storeResponse.getResult();
            ClerkDTO clerkDTO = new ClerkDTO();
            clerkDTO.setClerkName(clerkName);
            clerkDTO.setClerkCode(clerkCode);
            clerkDTO.setEnterpriseId(enterpriseId);
            clerkDTO.setStoreInfoId(storeInfoId);
            clerkDTO.setReason(ClerkLogReasonEnum.ORDER.getCode());
            clerkDTO.setRemark("订单号:" + orderNum);
//            clerkDTO.setStoreName(store.getStoreName());
            ServiceResponse serviceResponse = this.saveOrUpdate(clerkDTO);
            return serviceResponse;
//        } else {
//            logger.warn("门店不存在 enterpriseId:{}, storeId:{}", enterpriseId, storeInfoId);
//            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "门店不存在");
//        }
    }

    @Override
    public ServiceResponse<Integer> saveClerkForPosMember(String clerkName, String clerkCode, Integer storeInfoId, Integer enterpriseId, String cardNO) {
        ServiceResponse<StoreDTO> storeResponse = storeApiService.getStoreById(enterpriseId, storeInfoId);
        if (storeResponse.getResult() != null) {
            StoreDTO store = storeResponse.getResult();
            ClerkDTO clerkDTO = new ClerkDTO();
            clerkDTO.setClerkName(clerkName);
            clerkDTO.setClerkCode(clerkCode);
            clerkDTO.setEnterpriseId(enterpriseId);
            clerkDTO.setStoreInfoId(store.getStoreInfoId());
            clerkDTO.setReason(ClerkLogReasonEnum.MEMBER.getCode());
            clerkDTO.setRemark("卡号:" + cardNO);
            clerkDTO.setStoreName(store.getStoreName());
            ServiceResponse serviceResponse = this.saveOrUpdate(clerkDTO);
            return serviceResponse;
        } else {
            logger.warn("门店不存在 enterpriseId:{}, storeId:{}", enterpriseId, storeInfoId);
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "门店不存在");
        }
    }

    @Override
    public ServiceResponse<Integer> saveClerkForPosMember(PosAddClerkDTO posAddClerkDTO) {
        ServiceResponse<StoreDTO> storeResponse = storeApiService.getStoreById(posAddClerkDTO.getEnterpriseId(), posAddClerkDTO.getStoreInfoId());
        if (storeResponse.getResult() != null) {
            StoreDTO store = storeResponse.getResult();
            ClerkDTO clerkDTO = new ClerkDTO();
            clerkDTO.setClerkName(posAddClerkDTO.getClerkName());
            clerkDTO.setClerkCode(posAddClerkDTO.getClerkCode());
            clerkDTO.setEnterpriseId(posAddClerkDTO.getEnterpriseId());
            clerkDTO.setStoreInfoId(store.getStoreInfoId());
            clerkDTO.setReason(ClerkLogReasonEnum.MEMBER.getCode());
            clerkDTO.setRemark("卡号:" + posAddClerkDTO.getCardNO());
            clerkDTO.setStoreName(store.getStoreName());

            clerkDTO.setStatus((posAddClerkDTO.getOpenCardTime().getTime() + 15552000000L) > new Date().getTime() ? 1 : 0);

            ServiceResponse serviceResponse = this.saveOrUpdate(clerkDTO);
            return serviceResponse;
        } else {
            logger.warn("门店不存在 enterpriseId:{}, storeInfoId:{}", posAddClerkDTO.getEnterpriseId(), posAddClerkDTO.getStoreInfoId());
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "门店不存在");
        }
    }

    @Override
    public ServiceResponse<Integer> saveClerkByStoreInfoIdForOrder(String clerkName, String clerkCode, Integer storeInfoId, Integer enterpriseId, String orderNum) {
//        Integer storeId = this.storeService.getStoreIdByStoreInfoId(enterpriseId, storeInfoId);
//        if (storeId != null) {
            ClerkDTO clerkDTO = new ClerkDTO();
            clerkDTO.setClerkName(clerkName);
            clerkDTO.setClerkCode(clerkCode);
            clerkDTO.setEnterpriseId(enterpriseId);
            clerkDTO.setStoreInfoId(storeInfoId);
            clerkDTO.setReason(ClerkLogReasonEnum.ORDER.getCode());
            clerkDTO.setRemark("订单号:" + orderNum);

            ServiceResponse serviceResponse = this.saveOrUpdate(clerkDTO);
            return serviceResponse;
//        } else {
//            logger.warn("门店不存在 enterpriseId:{}, storeInfoId:{}", enterpriseId, storeInfoId);
//            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "门店不存在");
//        }
    }

    @Override
    public ServiceResponse<Integer> saveClerkByStoreInfoIdForOrder(OrderAddClerkDTO orderAddClerkDTO) {
        ClerkDTO clerkDTO = new ClerkDTO();
        clerkDTO.setClerkName(orderAddClerkDTO.getClerkName());
        clerkDTO.setClerkCode(orderAddClerkDTO.getClerkCode());
        clerkDTO.setEnterpriseId(orderAddClerkDTO.getEnterpriseId());
        clerkDTO.setStoreInfoId(orderAddClerkDTO.getStoreInfoId());
        clerkDTO.setReason(ClerkLogReasonEnum.ORDER.getCode());
        clerkDTO.setRemark("订单号:" + orderAddClerkDTO.getOrderNum());
        clerkDTO.setStatus((orderAddClerkDTO.getOrderTime().getTime() + 15552000000L) > new Date().getTime() ? 1 : 0);

        ServiceResponse serviceResponse = this.saveOrUpdate(clerkDTO);
        return serviceResponse;
    }

    @Override
    public ServiceResponse<ClerkDTO> getClerkByCode(Integer enterpriseId, String clerkCode, Integer storeId) {
        ServiceResponse<StoreDTO> storeResponse = storeApiService.getStoreById(enterpriseId, storeId);
        if (storeResponse.getResult() != null) {
            StoreDTO store = storeResponse.getResult();
            return getClerkByCodeAndStoreInfoId(enterpriseId, clerkCode, storeId);
        } else {
            logger.warn("门店不存在 enterpriseId:{}, storeId:{}", enterpriseId, storeId);
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "门店不存在");
        }
    }

    @Override
    public ServiceResponse<ClerkDTO> getClerkByCodeAndStoreInfoId(Integer enterpriseId, String clerkCode, Integer storeInfoId) {
        TabClerk clerk = this.clerkService.getClerkByCode(enterpriseId, clerkCode, storeInfoId);
        if (clerk != null) {
            return EnterpriseServiceResponse.success(EntityUtil.changeEntityByJSON(ClerkDTO.class, clerk));
        } else {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
    }

    private void storeLogByAddClerk(ClerkDTO clerkDTO) {
        try {
            StoreLogDTO storeLogDTO = new StoreLogDTO();
            storeLogDTO.setEnterpriseId(clerkDTO.getEnterpriseId());
            Integer storeId = this.storeService.getStoreIdByStoreInfoId(clerkDTO.getEnterpriseId(),
                    clerkDTO.getStoreInfoId());
            storeLogDTO.setStoreId(storeId);
            storeLogDTO.setContent("门店关联导购【导购编码" + clerkDTO.getClerkCode() + "】");
            storeLogDTO.setReason(clerkDTO.getReason());
            storeLogDTO.setLogType(StoreLogTypeEnum.ADD_CLERK.getCode());
            storeLogDTO.setRemark(clerkDTO.getRemark());
            storeLogDTO.setOperatorId(clerkDTO.getOperatorId());
            storeLogDTO.setOperatorName(clerkDTO.getOperatorName());

            storeLogService.saveStoreLog(storeLogDTO);
        } catch (Exception e) {
            logger.warn("保存导购日志出错", e);
        }
    }

    private boolean validStoreIsOther(Integer enterpriseId, Integer storeInfoId) {
        Integer owner = storeService.getStoreOwnerByStoreInfoId(enterpriseId, storeInfoId);
        return owner == StoreOwnTypeEnum.OTHER.getCode();
    }

    @Override
    @SuppressWarnings("unchecked")
    public ServiceResponse<Page<ClerkStoreListDTO>> listClerk(Integer enterpriseId, ClerkSearchDTO clerkSearchDTO,
            Integer pageNum, Integer pageSize) {

        boolean hasStoreIds = StringUtils.isNotBlank(clerkSearchDTO.getStoreIds());
        //如果有门店数据条件，则不判断门店分组数据，因为分店分组数据也是为了获取门店
        if (!hasStoreIds) {
            if (clerkSearchDTO.getStoreGroupId() == null) {
                return ServiceResponse.failure(ErrorCode.MISS_PARAMETER.getCode(), ErrorCode.MISS_PARAMETER.getMsg());
            }
        }
        // 分组的id
        List<TabStoreGroup> groupList = storeGroupService.listStoreGroupAndChildren(clerkSearchDTO.getStoreGroupId());
        List<Integer> groupIds = groupList.stream().map(TabStoreGroup::getStoreGroupId).collect(Collectors.toList());
        //         查询条件对应的storeId
        String storeIdByClerkSearch;
        if (StringUtils.isNotBlank(clerkSearchDTO.getSearch())) {
            List<Integer> storeInfoIdsBySearch = clerkService.getStoreInfoIdsBySearch(enterpriseId, clerkSearchDTO.getSearch());
            if (CollectionUtils.isEmpty(groupIds) || CollectionUtils.isEmpty(storeInfoIdsBySearch)) {
                return ServiceResponse.success(new Page<>());
            } else {
                storeIdByClerkSearch = getStoreIdByStoreInfoIds(storeInfoIdsBySearch, null);
            }
        } else {
            storeIdByClerkSearch = "";
        }
        clerkSearchDTO.setEnterpriseId(enterpriseId);
        if (!hasStoreIds) {
            clerkSearchDTO.setStoreIds(storeIdByClerkSearch);
        }
        ServiceResponse<Page<StoreDTO>> serviceResponse = listStoreIds(clerkSearchDTO, pageNum, pageSize);
        if (!serviceResponse.isSuccess()) {
            return ServiceResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
        Page<StoreDTO> page = serviceResponse.getResult();
        if (page == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "ES门店数据为空");
        }
        List<StoreDTO> storeList = page.getResult();
        List<Integer> storeInfoIds = new ArrayList<>(storeList.size());
        List<ClerkStoreListDTO> storeClerkDTOList = new ArrayList<>(storeList.size());
        for (StoreDTO store : storeList) {
            ClerkStoreListDTO clerkStoreListDTO = new ClerkStoreListDTO();
            clerkStoreListDTO.setStoreId(store.getStoreId());
            clerkStoreListDTO.setStoreInfoId(store.getStoreInfoId());
            clerkStoreListDTO.setStoreName(store.getStoreName());
            clerkStoreListDTO.setStoreCode(store.getStoreCode());
            clerkStoreListDTO.setOwnType(store.getOwnType());
            storeClerkDTOList.add(clerkStoreListDTO);
            storeInfoIds.add(store.getStoreInfoId());
        }
        List<TabClerk> clerkList = clerkService.listClerkByStoreInfoId(enterpriseId, storeInfoIds,
                clerkSearchDTO.getSearch(), clerkSearchDTO.getClerkType());
        if (CollectionUtils.isNotEmpty(clerkList)) {
            for (TabClerk clerk : clerkList) {
                if (Integer.valueOf(1).equals(clerk.getClerkType())) {
                    clerk.setPositionName("店长");
                }
            }
        }

        Map<Integer, List<TabClerk>> storeIdMap = CollectionUtil.group(clerkList, "storeInfoId");
        for (ClerkStoreListDTO clerkStoreListDTO : storeClerkDTOList) {
            List<TabClerk> tabClerks = storeIdMap.get(clerkStoreListDTO.getStoreInfoId());
            if (CollectionUtils.isNotEmpty(tabClerks)) {
                clerkStoreListDTO.setClerkList(EntityUtil.changeEntityListByOrika(ClerkDTO.class, tabClerks));
            } else {
                clerkStoreListDTO.setClerkList(Collections.emptyList());
            }
        }
        Page resultPage = new Page();
        resultPage.setPageSize(page.getPageSize());
        resultPage.setCurrentPage(page.getCurrentPage());
        resultPage.setTotalCount(page.getTotalCount());
        resultPage.setTotalPage(page.getPages());
        resultPage.setResult(storeClerkDTOList);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<Integer> getTotalClerk(Integer enterpriseId, ClerkSearchDTO clerkSearchDTO) {
        boolean hasStoreIds = StringUtils.isNotBlank(clerkSearchDTO.getStoreIds());
        //如果有门店数据条件，则不判断门店分组数据，因为分店分组数据也是为了获取门店
        if (!hasStoreIds) {
            if (clerkSearchDTO.getStoreGroupId() == null) {
                return ServiceResponse.failure(ErrorCode.MISS_PARAMETER.getCode(), ErrorCode.MISS_PARAMETER.getMsg());
            }
        }
        // 分组的id
        List<TabStoreGroup> groupList = storeGroupService.listStoreGroupAndChildren(clerkSearchDTO.getStoreGroupId());
        List<Integer> groupIds = groupList.stream().map(TabStoreGroup::getStoreGroupId).collect(Collectors.toList());
        //         查询条件对应的storeId
        String storeIdByClerkSearch;
        if (StringUtils.isNotBlank(clerkSearchDTO.getSearch())) {
            List<Integer> storeInfoIdsBySearch = clerkService.getStoreInfoIdsBySearch(enterpriseId, clerkSearchDTO.getSearch());
            if (CollectionUtils.isEmpty(groupIds) || CollectionUtils.isEmpty(storeInfoIdsBySearch)) {
                storeIdByClerkSearch = "";
            } else {
                storeIdByClerkSearch = getStoreIdByStoreInfoIds(storeInfoIdsBySearch, null);
            }
        } else {
            storeIdByClerkSearch = "";
        }
        clerkSearchDTO.setEnterpriseId(enterpriseId);
        if (!hasStoreIds) {
            clerkSearchDTO.setStoreIds(storeIdByClerkSearch);
        }
        ServiceResponse<Page<StoreDTO>> serviceResponse = listStoreIds(clerkSearchDTO, 1, Integer.MAX_VALUE);
        if (!serviceResponse.isSuccess()) {
            return ServiceResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
        Page<StoreDTO> page = serviceResponse.getResult();
        if (page == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "ES门店数据为空");
        }
        List<Integer> storeInfoIds = page.getResult().stream().map(StoreDTO::getStoreInfoId)
                .collect(Collectors.toList());
        Integer totalClerk = clerkService.getTotalClerk(enterpriseId, storeInfoIds, clerkSearchDTO.getSearch());

        return ServiceResponse.success(totalClerk);
    }

//    private String getStoreIdByStoreInfoIds(List<Integer> storeInfoIdsBySearch, String search) {
//        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
//        storeSearchDTO.setStoreInfoIds(StringUtils.join(storeInfoIdsBySearch, " "));
//        storeSearchDTO.setSearch(search);
//        Page<StoreDTO> page = storeApiService.listStore(storeSearchDTO, 1, Integer.MAX_VALUE).getResult();
//        if (page == null) {
//            return "";
//        } else {
//            return Optional.ofNullable(page.getResult()).orElse(Collections.emptyList()).stream().map(e -> e.getStoreId().toString())
//                    .reduce((x, y) -> x + " " + y).orElse("");
//        }
//    }

    private String getStoreIdByStoreInfoIds(List<Integer> storeInfoIdsBySearch, String search) {
        StoreSearchDBDTO storeSearchDBDTO = new StoreSearchDBDTO();
        storeSearchDBDTO.setStoreInfoIdList(storeInfoIdsBySearch);
        storeSearchDBDTO.setSearch(search);
        List<StoreDTO> dtos = storeService.listStore(storeSearchDBDTO, 1, Integer.MAX_VALUE);
        return Optional.ofNullable(dtos).orElse(Collections.emptyList()).stream().map(e -> e.getStoreId().toString())
                .reduce((x, y) -> x + " " + y).orElse("");
    }


    @SuppressWarnings("unchecked")
    private ServiceResponse<Page<StoreDTO>> listStoreIds(ClerkSearchDTO clerkSearchDTO, Integer pageNum,
            Integer pageSize) {
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        storeSearchDTO.setEnterpriseId(clerkSearchDTO.getEnterpriseId());
        storeSearchDTO.setStoreGroupId(clerkSearchDTO.getStoreGroupId());
        storeSearchDTO.setSearchJson(clerkSearchDTO.getSearchJson());
        storeSearchDTO.setStoreIds(clerkSearchDTO.getStoreIds());

        return storeApiService.listStore(storeSearchDTO, pageNum, pageSize,
                "storeId,storeName,storeCode,storeInfoId,ownType");
    }

    @Override
    public ServiceResponse updateClerkStatus(Integer enterpriseId, String clerkIds, Integer status) {
        return ServiceResponse.success(clerkService.updateClerkStatus(enterpriseId, clerkIds, status));
    }

    @Override
    public ServiceResponse<List<ClerkDTO>> listClerkByIds(Integer enterpriseId, String clerkIds) {
        List<TabClerk> clerkList = clerkService.listClerkByIds(enterpriseId, clerkIds);
        List<ClerkDTO> clerkDTOList = EntityUtil.changeEntityListByJSON(ClerkDTO.class, clerkList);
        return EnterpriseServiceResponse.success(clerkDTOList);
    }

    @Override
    public ServiceResponse<List<ClerkDTO>> listClerkByStoreId(Integer enterpriseId, Integer storeId, String search) {
        ServiceResponse<StoreDTO> response = storeApiService.getStoreById(enterpriseId, storeId);
        if (response.isSuccess() && response.getResult() != null) {
            List<Integer> storeInfoIds = new ArrayList<>();
            storeInfoIds.add(response.getResult().getStoreInfoId());
            List<TabClerk> clerkList = this.clerkService.listClerkByStoreInfoId(enterpriseId, storeInfoIds, search);
            return ServiceResponse.success(EntityUtil.changeEntityListByJSON(ClerkDTO.class, clerkList));
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<ClerkDTO> getHeadClerk(Integer enterpriseId, Integer storeId) {
        ServiceResponse<StoreDTO> response = storeApiService.getStoreById(enterpriseId, storeId);
        if (response.isSuccess() && response.getResult() != null) {
            List<Integer> storeInfoIds = new ArrayList<>();
            storeInfoIds.add(response.getResult().getStoreInfoId());
            List<TabClerk> clerkList = clerkService.listClerkByStoreInfoId( enterpriseId, storeInfoIds, null, 1);
            if (CollectionUtils.isNotEmpty(clerkList)) {
                return ServiceResponse.success(EntityUtil.changeEntityNew(ClerkDTO.class, clerkList.get(0)));
            }
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<ClerkDTO> getClerkById(Integer enterpriseId, Integer clerkId) {
        TabClerk tabClerk = this.clerkService.getById(clerkId);
        return ServiceResponse.success(EntityUtil.changeEntityByJSON(ClerkDTO.class, tabClerk));
    }

    @Override
    public ServiceResponse<List<ClerkDTO>> listClerk(Integer enterpriseId, String search) {
        List<TabClerk> clerkList = this.clerkService.listClerkByStoreInfoId(enterpriseId, search);
        return ServiceResponse.success(EntityUtil.changeEntityListByJSON(ClerkDTO.class, clerkList));
    }

    @Override
    public ServiceResponse<String> importClerkForOperation(ProcessBatchClerkDTO clerkInfo) {
        if (clerkInfo == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "对象为空");
        }
        logger.info("批量处理导购的参数：{}", JSONObject.toJSONString(clerkInfo));
        //param valid
        Integer regionId = clerkInfo.getRegionId();
        Integer enterpriseId = clerkInfo.getEnterpriseId();
        //商户ID校验
        ServiceResponse enterpriseResult = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (!enterpriseResult.isSuccess()) {
            return responseErrorElement(ImportClerkBatchResultEnum.E1);
        }
        if (regionId == null) {
            return responseErrorElement(ImportClerkBatchResultEnum.A1);
        }
        TabStoreRegion tabStoreRegion = storeRegionService.getById(enterpriseId, regionId);
        if (tabStoreRegion == null) {
            return responseErrorElement(ImportClerkBatchResultEnum.A2);
        }
        String storeCode = clerkInfo.getStoreCode();
        if (StringUtils.isBlank(storeCode)) {
            return responseErrorElement(ImportClerkBatchResultEnum.A3);
        }
        StoreDTO storeDTO = storeService.getStoreByRegionIdAndStoreCode(enterpriseId, tabStoreRegion.getRegionId(),
                clerkInfo.getStoreCode());
        if (storeDTO == null) {
            return responseErrorElement(ImportClerkBatchResultEnum.A4);
        }
        Integer storeInfoId = storeDTO.getStoreInfoId();
        String clerkCode = clerkInfo.getClerkCode();
        if (StringUtils.isBlank(clerkCode)) {
            return responseErrorElement(ImportClerkBatchResultEnum.C1);
        }

        if (StringUtils.isBlank(clerkInfo.getClerkName())) {
            return responseErrorElement(ImportClerkBatchResultEnum.C3);
        }
        String nationcode = clerkInfo.getNationcode();
        if (StringUtils.isBlank(nationcode)) {
            nationcode = Constants.NATION_CODE;
        }
        Integer gender = clerkInfo.getClerkGender();
        if (gender != null && gender.intValue() != 1 && gender.intValue() != 2) {
            return responseErrorElement(ImportClerkBatchResultEnum.D2);
        }
        String phone = clerkInfo.getPhoneNumber();
        if (StringUtils.isBlank(phone)) {
            return responseErrorElement(ImportClerkBatchResultEnum.D4);
        }
        if (clerkInfo.getClerkType() == null) {
            return responseErrorElement(ImportClerkBatchResultEnum.D5);
        }
        if (clerkService.isRepeatByPhoneNumber(storeInfoId, phone, null, nationcode)) {
            return responseErrorElement(ImportClerkBatchResultEnum.D3);
        }
        ClerkDTO clerkDTO = EntityUtil.changeEntityNew(ClerkDTO.class, clerkInfo);

        TabClerk clerk = this.clerkService.getClerkByCode(enterpriseId, clerkCode, storeInfoId);
        boolean isAdd = true;
        //如果代购code存在，则进行更新操作
        if (clerk != null) {
            clerkDTO.setClerkId(clerk.getClerkId());
            //return responseErrorElement(ImportClerkBatchResultEnum.C2);
            isAdd = false;

        }

        clerkDTO.setStoreInfoId(storeInfoId);
        clerkDTO.setStoreName(storeDTO.getStoreName());

        ServiceResponse serviceResponse = saveOrUpdate(clerkDTO);
        if (!serviceResponse.isSuccess()) {
            return serviceResponse;
        }
        Integer id = (Integer) serviceResponse.getResult();
        if (isAdd) {
            clerkLogService.saveClerkLogBySaveClerk(enterpriseId, id, clerkDTO.getReason(),
                    clerkDTO.getRemark(), clerkDTO.getOperatorId(), clerkDTO.getOperatorName());
            storeLogByAddClerk(clerkDTO);
        } else {
            id = clerk.getClerkId();
            clerkDTO.setClerkId(id);
            clerkLogService.saveClerkLogByUpdateClerk(clerk, clerkDTO, clerkDTO.getEnterpriseId(),
                    clerkDTO.getClerkId(), clerkDTO.getReason(), clerkDTO.getRemark(), clerkDTO.getOperatorId(),
                    clerkDTO.getOperatorName());
        }
        //性别为空，但是标记导入
        if (gender == null) {
            return responseErrorElement(ImportClerkBatchResultEnum.D1);
        }
        return ServiceResponse.success(id.toString());
    }

    @Override
    public ServiceResponse<Integer> addClerkToOtherStore(Integer clerkId, Integer storeInfoId, Integer enterpriseId) {
        logger.info("clerkId:{}, storeInfoId:{}, enterpriseId:{}", clerkId, storeInfoId, enterpriseId);
        TabClerk clerk = this.clerkService.getById(clerkId);
        if (clerk == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "导购不存在");
        }
        StoreDTO store = this.storeApiService.getStoreByStoreInfoId(enterpriseId, storeInfoId).getResult();
        if (store == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "门店不存在");
        }
        if (clerk.getStoreInfoId().equals(store.getStoreInfoId())) {
            logger.info("导购[clerkId:{}]已存在门店[storeId:{}, storeInfoId:{}]下", clerkId, store.getStoreId(), store.getStoreInfoId());
            return EnterpriseServiceResponse.success(clerkId);
        }
        ClerkDTO clerkDTO = EntityUtil.changeEntityByJSON(ClerkDTO.class, clerk);
        clerkDTO.setClerkId(null);
        clerkDTO.setStoreInfoId(store.getStoreInfoId());
        ServiceResponse<Integer> clerkResp = this.saveOrUpdate(clerkDTO);
        if (clerkResp.isSuccess()) {
            logger.info("导购[clerkId:{}]不存在门店[storeId:{}, storeInfoId:{}]下, 新增clerkId：{}", clerkId, store.getStoreId(), store.getStoreInfoId(), clerkResp.getResult());
            return EnterpriseServiceResponse.success(clerkResp.getResult());
        } else {
            logger.info("保存导购失败：{}", clerkResp.getMessage());
            return EnterpriseServiceResponse.failure(clerkResp.getCode(), clerkResp.getMessage());
        }
    }

    @Override
    public ServiceResponse<Boolean> ensureClerkInStore(Integer clerkId, Integer storeInfoId, Integer enterpriseId) {
        logger.info("clerkId:{}, storeInfoId:{}, enterpriseId:{}", clerkId, storeInfoId, enterpriseId);
        TabClerk clerk = this.clerkService.getById(clerkId);
        if (clerk == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "导购不存在", Boolean.FALSE);
        }
        StoreDTO store = this.storeApiService.getStoreById(enterpriseId, storeInfoId).getResult();
        if (store == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "门店不存在", Boolean.FALSE);
        }
        if (clerk.getStoreInfoId().equals(store.getStoreInfoId())) {
            logger.info("导购[clerkId:{}]已存在门店[storeId:{}, storeInfoId:{}]下", clerkId, store.getStoreId(), store.getStoreInfoId());
            return EnterpriseServiceResponse.success(Boolean.TRUE);
        } else {
            logger.info("导购[clerkId:{}]不存在门店[storeId:{}, storeInfoId:{}]下", clerkId, store.getStoreId(), store.getStoreInfoId());
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "导购不存在该门店下", Boolean.FALSE);
        }
    }

    @Override
    public ServiceResponse<List<ClerkDTO>> listClerkByStoreBrand(Integer enterpriseId, Integer storeBrandId, String clerkName) {
        StoreSearchDBDTO storeSearchDBDTO = new StoreSearchDBDTO();
        storeSearchDBDTO.setEnterpriseId(enterpriseId);
        storeSearchDBDTO.setStoreBrandId(storeBrandId);
        List<StoreDTO> list = this.storeService.listStore(storeSearchDBDTO, 1, 10000);
        if (CollectionUtils.isEmpty(list)) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
        // list 如果太大， 让 mybatis 解析 会打满内存
        String storeInfoIds = list.stream().map(e -> e.getStoreInfoId().toString()).reduce((x, y) -> x + "," + y).get();
        storeInfoIds = "(" + storeInfoIds + ")";
        List<TabClerk> clerkList = this.clerkService.listClerkByStoreInfoId(enterpriseId, storeInfoIds, clerkName);
        List<ClerkDTO> dtoList = EntityUtil.changeEntityListByJSON(ClerkDTO.class, clerkList);
        return EnterpriseServiceResponse.success(dtoList);
    }

    @Override
    public ServiceResponse updateClerkLeader(Integer clerkId) {
        TabClerk clerk = this.clerkService.getById(clerkId);
        clerk.setClerkType(1);
        TabClerk oldClerk = this.clerkService.getClerkLeaderByStoreInfoId(clerk.getEnterpriseId(), clerk.getStoreInfoId());
        oldClerk.setClerkType(0);

        this.clerkService.update(clerk);
        this.clerkService.update(oldClerk);

        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> syncClerkToWeimob(Integer wmMallStoreId, List<Integer> storeIdList, Integer enterpriseId) {
        ClerkSearchDTO searchDTO = new ClerkSearchDTO();
        searchDTO.setEnterpriseId(enterpriseId);
        StringBuilder sb = new StringBuilder();
        storeIdList.forEach(e -> sb.append(e).append(","));
        searchDTO.setStoreIds(sb.toString());
        ServiceResponse<Page<ClerkStoreListDTO>> pageResult = listClerk(enterpriseId, searchDTO, 0, storeIdList.size());
        if (pageResult.isSuccess()) {
            Page<ClerkStoreListDTO> page = pageResult.getResult();
            if (page != null) {
                List<ClerkStoreListDTO> list = page.getResult();
                if (CollectionUtils.isNotEmpty(list)) {
                    list.forEach(e -> {
                        List<ClerkDTO> clerkList = e.getClerkList();
                        if (CollectionUtils.isNotEmpty(clerkList)) {
                            clerkList.forEach(clerk -> {
                                WeimobGuiderSynDTO weimobGuiderSynDTO = new WeimobGuiderSynDTO();
                                weimobGuiderSynDTO.setStoreNumber(e.getStoreCode());
                                weimobGuiderSynDTO.setClerkId(clerk.getClerkId());
                                weimobGuiderSynDTO.setGuiderName(clerk.getClerkName());
                                weimobGuiderSynDTO.setGuiderPhone(clerk.getPhoneNumber());
                                weimobGuiderSynDTO.setJobNumber(clerk.getClerkCode());
                                weimobGuiderSynDTO.setStoreInfoId(e.getStoreInfoId());
                                asynClerkInfoToWeimobMQ(wmMallStoreId, enterpriseId, weimobGuiderSynDTO);
                            });
                        }
                    });
                }
            }
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> syncClerkToWeimobTask(String param) {
        JSONObject jsonObject = JSONObject.parseObject(param);
        String enterpriseId = jsonObject.getString("enterpriseId");
        String wmMallStoreId = jsonObject.getString("wmMallStoreId");
        String wmGuiderDTOStr = (String) jsonObject.get("wmGuiderDTO");
        WeimobGuiderSynDTO wmGuiderDTO = JSONObject.parseObject(wmGuiderDTOStr, WeimobGuiderSynDTO.class);
        ServiceResponse<Void> result = weimobStoreSiteService.addWeimobGuider(Integer.parseInt(wmMallStoreId),
                Integer.parseInt(enterpriseId), wmGuiderDTO);
        logger.info("异步导购结果：code:{}, message:{}", result.getCode(), result.getMessage());
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> syncClerkToWeimob(Integer wmMallStoreId, String clerkIds, Integer enterpriseId) {
        if (StringUtils.isBlank(clerkIds)) {
            return ServiceResponse.success();
        }
        List<TabClerk> clerkList = clerkService.listClerkByIds(enterpriseId, clerkIds);
        if (CollectionUtils.isNotEmpty(clerkList)) {
            List<Integer> storeInfoIdList = clerkList.stream().map(e -> e.getStoreInfoId()).collect(Collectors.toList());
            Map<String, String> storeInfoIdMapStoreCode = new HashMap<>(16);
            //查询门店，为了获取门店code
            if (CollectionUtils.isNotEmpty(storeInfoIdList)) {
                StoreSearchDBDTO storeSearchDBDTO = new StoreSearchDBDTO();
                storeSearchDBDTO.setStoreInfoIdList(storeInfoIdList);
                com.github.pagehelper.Page<StoreDTO> storePage = storeService.listStore(storeSearchDBDTO, 0, storeInfoIdList.size());
                if (storePage != null) {
                    List<StoreDTO> storeList = storePage.getResult();
                    if (CollectionUtils.isNotEmpty(storeList)) {
                        storeList.forEach( store -> storeInfoIdMapStoreCode.put(store.getStoreInfoId().toString(), store.getStoreName()));
                    }
                }
            }
            clerkList.forEach(clerkTemp -> {
                WeimobGuiderSynDTO weimobGuiderSynDTO = new WeimobGuiderSynDTO();
                weimobGuiderSynDTO.setStoreNumber(storeInfoIdMapStoreCode.get(clerkTemp.getStoreInfoId().toString()));
                weimobGuiderSynDTO.setClerkId(clerkTemp.getClerkId());
                weimobGuiderSynDTO.setGuiderName(clerkTemp.getClerkName());
                weimobGuiderSynDTO.setGuiderPhone(clerkTemp.getPhoneNumber());
                weimobGuiderSynDTO.setJobNumber(clerkTemp.getClerkCode());
                weimobGuiderSynDTO.setStoreInfoId(clerkTemp.getStoreInfoId());
                asynClerkInfoToWeimobMQ(wmMallStoreId, enterpriseId, weimobGuiderSynDTO);
            });

        }
        return ServiceResponse.success();
    }

    private void asynClerkInfoToWeimobMQ(Integer wmMallStoreId, Integer enterpriseId, WeimobGuiderSynDTO wmGuiderDTO) {
        GicMQClient client = GICMQClientUtil.getClientInstance();
        Map<String, Object> params = new HashMap<>(4);
        params.put("enterpriseId", enterpriseId);
        params.put("wmMallStoreId", wmMallStoreId);
        params.put("wmGuiderDTO", JSONObject.toJSONString(wmGuiderDTO));

        logger.info("异步导购微盟放入参数:{}", JSONObject.toJSONString(params));
        try {
            client.sendMessage("tmpClerkToWeimob", JSONObject.toJSONString(params));
        } catch (Exception e) {
            logger.warn("同步导购到微盟错误:{}", e.getMessage(), e);
        }
    }

    private static ServiceResponse<String> responseErrorElement(ImportClerkBatchResultEnum error) {
        logger.warn("批处理导购错误返回结果:{}", error.toString());
        return ServiceResponse.failure(error.getCode(), error.toString());
    }

}