package com.gic.store.service.outer.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.store.constant.*;
import com.gic.store.dto.ClerkDTO;
import com.gic.store.dto.ClerkTmpDTO;
import com.gic.store.dto.StoreDTO;
import com.gic.store.entity.TabClerkTmp;
import com.gic.store.service.ClerkApiService;
import com.gic.store.service.ClerkImportService;
import com.gic.store.service.ClerkTaskApiService;
import com.gic.store.service.StoreService;
import com.task.allocation.api.AbstractTaskAllocationOperation;
import com.task.allocation.exception.TaskAllocationException;
import com.task.allocation.qo.InitTaskQO;
import com.task.allocation.util.TaskAllocationSdkClient;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author zhiwj
 * @date 2019/7/17
 */
@Service("clerkTaskApiService")
public class ClerkTaskServiceImpl extends AbstractTaskAllocationOperation implements ClerkTaskApiService {

    private static final Logger logger = LogManager.getLogger(ClerkTaskServiceImpl.class);

    @Autowired
    private ClerkImportService clerkImportService;

    @Autowired
    private ClerkApiService clerkApiService;

    @Autowired
    private StoreService storeService;

    @Override
    public String importDataToClerk(Integer enterpriseId, String userId, String uuId) {
        logger.info("enterpriseId:{}, userId:{}, uuId:{}", enterpriseId, userId, uuId);
        String message = "";
        InitTaskQO initTaskQo = new InitTaskQO();
        initTaskQo.setEnterpriseId(enterpriseId.toString());// 企业id
        initTaskQo.setOperationUserId(userId);// 操作人id
        // key
        initTaskQo.setTaskMqKey(Constants.CLERK_BATCH_IMPORT_MQ_KEY);// 注册的队列名称
        initTaskQo.setTaskType(Constants.CLERK_BATCH_IMPORT_TASK_TYPE);// 任务类型
        initTaskQo.setParams(enterpriseId.toString());// 你自己处理需要的参数
        initTaskQo.setTaskSignKey(uuId);// 任务类型 唯一标识符
        try {
            TaskAllocationSdkClient instance = TaskAllocationSdkClient.getInstance();// 初始化sdk
            instance.initTask(initTaskQo);// 初始化接口
            logger.info("导购批量导入队列初始化成功");
        } catch (Exception e) {
            logger.warn("导购批量导入队列初始化异常", e);
            message = "导购批量导入队列初始化异常";
        }
        return message;
    }

    @Override
    public List<Object> getListTasks(String params) {
        List<TabClerkTmp> list = clerkImportService.listUnImportClerk(params);
        logger.info("导购批量导入放入队列数据数量：{}", list.size());
        List listTask = EntityUtil.changeEntityListByOrika(ClerkTmpDTO.class, list);
        return listTask;
    }

    @Override
    public void dealSingle(Object obj) throws TaskAllocationException {
        logger.info("开始执行队列");
        ClerkTmpDTO t = (ClerkTmpDTO) obj;
        try {
            this.importDataToStore(t);
        } catch (Exception e) {
            logger.warn("写入门成员表异常", e);
            t.setErrorMessage("导入异常");
            clerkImportService.updateData(t);
        }
    }

    private void importDataToStore(ClerkTmpDTO t) {
        StoreDTO store = storeService.getByStoreNameAndStoreCode(t.getEnterpriseId(), t.getStoreName(), t.getStoreCode());
        if (store == null) {
            t.setErrorMessage("对应的门店不存在");
            clerkImportService.updateData(t);
            return;
        }
        if (StoreOwnTypeEnum.OWNER.getCode() != store.getOwnType()) {
            t.setErrorMessage("不能在被授权门店下导入导购");
            clerkImportService.updateData(t);
            return;
        }

        ClerkDTO clerkDTO = new ClerkDTO();
        clerkDTO.setEnterpriseId(t.getEnterpriseId());
        clerkDTO.setStoreInfoId(store.getStoreInfoId());
        clerkDTO.setClerkName(t.getClerkName().trim());
        clerkDTO.setClerkCode(t.getClerkCode().trim());
        clerkDTO.setClerkGender(GenderEnum.parseMessage(t.getClerkGender().trim()).getCode());
        clerkDTO.setNationcode(t.getNationcode().trim());
        clerkDTO.setPhoneNumber(t.getPhoneNumber().trim());
        clerkDTO.setPositionName(t.getPositionName().trim());
        clerkDTO.setClerkType(StringUtils.equals(t.getClerkType(), "是")? 1 : 0);

        clerkDTO.setReason(ClerkLogReasonEnum.WEB.getCode());
        clerkDTO.setRemark("批量导入");
        clerkDTO.setOperatorId(t.getOperatorId());
        clerkDTO.setOperatorName(t.getOperatorName());

        ServiceResponse serviceResponse = clerkApiService.saveOrUpdate(clerkDTO);
        String msg;
        if (serviceResponse.isSuccess()) {
            msg = StoreImportEnum.SUCCESS.getCode();
        } else {
            msg = serviceResponse.getMessage();
        }
        t.setErrorMessage(msg);
        clerkImportService.updateData(t);
    }
}
