package com.gic.store.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.AuthModeEnum;
import com.gic.auth.constant.AuthorizationStatusEnum;
import com.gic.auth.service.UnionEnterpriseApiService;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.commons.util.ToolUtil;
import com.gic.mq.sdk.GicMQClient;
import com.gic.store.constant.StoreGroupConstant;
import com.gic.store.constant.StoreOwnTypeEnum;
import com.gic.store.dto.StoreDTO;
import com.gic.store.dto.StoreSearchDTO;
import com.gic.store.dto.StoreWidgetDTO;
import com.gic.store.entity.TabAuthStoreLog;
import com.gic.store.entity.TabStoreGroup;
import com.gic.store.service.*;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service("storeAuthorizationApiService")
public class StoreAuthorizationApiServiceImpl implements StoreAuthorizationApiService {
    private final static Logger log = LogManager.getLogger(StoreAuthorizationApiServiceImpl.class);
    private final static int pageSize = 1000;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;
    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private StoreStrategyService storeStrategyService;
    @Autowired
    private StoreGroupService storeGroupService;
    @Autowired
    private StoreService storeService;
    @Autowired
    private UnionEnterpriseApiService unionEnterpriseApiService;
    @Autowired
    private StoreRegionApiService storeRegionApiService;
    @Autowired
    private StoreDictApiService storeDictApiService;
    @Autowired
    private AuthStoreLogService authStoreLogService;

    @Override
    public ServiceResponse<Void> authStore(Integer storeResourceId, Integer toEnterpriseId, Integer enterpriseId) {
        ServiceResponse<StoreWidgetDTO> storeWidget = this.storeWidgetApiService.getStoreWidget(storeResourceId);
        if(storeWidget.isSuccess()){
            StoreWidgetDTO storeWidgetDTO = storeWidget.getResult();
            if(storeWidgetDTO != null){
                Integer authMode = storeWidgetDTO.getAuthMode();
                if(authMode == AuthModeEnum.NO.getCode()){
                    this.unionEnterpriseApiService.updateStoreAuthorizationStatus(AuthorizationStatusEnum.NO.getCode(),
                            enterpriseId, AuthorizationStatusEnum.NO.getMessage(), null);
                    return ServiceResponse.success();
                }
                StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
                storeSearchDTO.setEnterpriseId(enterpriseId);
                storeSearchDTO.setSearchJson(storeWidgetDTO.getSearchParam());
                Long count = this.storeApiService.queryStoreCountFromES(storeSearchDTO).getResult();
                if(count > 0){
                    this.queryStoreByPage(count, storeSearchDTO, toEnterpriseId, enterpriseId, null);
                }else {
                    this.unionEnterpriseApiService.updateStoreAuthorizationStatus(AuthorizationStatusEnum.SUCCESS.getCode(),
                            enterpriseId, "授权成功", null);
                }
            }
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> reAuthStore(String key) {
        List<TabAuthStoreLog> list = this.authStoreLogService.listFailedStore(key);
        StringBuilder storeInfoIds = new StringBuilder();
        if(CollectionUtils.isNotEmpty(list)){
            Integer fromEnterpriseId = list.get(0).getFromEnterpriseId();
            Integer toEnterpriseId = list.get(0).getToEnterpriseId();
            for(TabAuthStoreLog log : list){
                storeInfoIds.append(log.getStoreInfoId()).append(",");
            }
            StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
            storeSearchDTO.setEnterpriseId(toEnterpriseId);
            storeSearchDTO.setStoreInfoIds(storeInfoIds.toString());
            this.queryStoreByPage(Long.valueOf(list.size()+""), storeSearchDTO, toEnterpriseId, fromEnterpriseId, key);
        }
        return null;
    }


    @Override
    public ServiceResponse<Void> authorizationSingleStore(String params){
        log.info("authorizationSingleStore:{}", params);
        JSONObject json = JSON.parseObject(params);
        StoreDTO storeDTO = JSON.toJavaObject(json.getJSONObject("store"), StoreDTO.class);
        Integer toEnterpriseId = json.getInteger("toEnterpriseId");
        Integer fromEnterpriseId = json.getInteger("enterpriseId");
        Integer isEnd = json.getInteger("isEnd");
        String key = json.getString("key");
        storeDTO.setEnterpriseId(toEnterpriseId);
        storeDTO.setFromEnterpriseId(fromEnterpriseId);
        int logId = this.authStoreLogService.saveAuthStoreLog(storeDTO.getStoreInfoId(), fromEnterpriseId, toEnterpriseId, key);
        this.saveStore(storeDTO, logId);
        if(isEnd == 1){
            int count = this.authStoreLogService.getCount(key, 2);
            int total = this.authStoreLogService.getCount(key, null);
            if(count > 0){
                this.unionEnterpriseApiService.updateStoreAuthorizationStatus
                        (AuthorizationStatusEnum.PARTSUCCESS.getCode(), storeDTO.getEnterpriseId(), "成功授权"+(total-count)+"条数据，失败"+count+"条", key);
            }
            this.unionEnterpriseApiService.updateStoreAuthorizationStatus(AuthorizationStatusEnum.SUCCESS.getCode(), storeDTO.getEnterpriseId(), "授权成功", key);
        }
        return null;
    }

    @Override
    public ServiceResponse<Void> saveStore(StoreDTO storeDTO, Integer logId){
        String groupId = storeStrategyService.isHitStrategy(storeDTO, StoreGroupConstant.STORE_GROUP_STRATEGY_TYPE);
        if (StringUtils.isBlank(groupId)) {
            TabStoreGroup group = storeGroupService.selectUnGroupedStore(storeDTO.getEnterpriseId());
            if(group != null){
                groupId = group.getStoreGroupId()+"";
                storeDTO.setStoreGroupId(Integer.valueOf(groupId));
            }
        }
        storeDTO.setIsEditStoreGroup(0);
        storeDTO.setOwnType(StoreOwnTypeEnum.OTHER.getCode());
        int i = this.storeService.authAddStore(storeDTO);
        log.info("{}，门店授权结果：{}", storeDTO.getStoreInfoId(), i);
        if(i == 0 && logId != 0){
            this.authStoreLogService.updateStatus(logId, 2);
        }else {
            this.authStoreLogService.updateStatus(logId, 1);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> saveStore(StoreDTO storeDTO) {
        return this.saveStore(storeDTO, 0);
    }

    @Override
    public ServiceResponse<Void> cancelStore(Integer storeId, Integer enterpriseId){
        List<Integer> list = new ArrayList<>();
        list.add(storeId);
        this.storeService.cancelAuthStore(list, enterpriseId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> cancelStoreAll(Integer fromEnterpriseId, Integer enterpriseId) {
        this.storeService.cancelAuthStoreAll(fromEnterpriseId, enterpriseId);
        return ServiceResponse.success();
    }

    private void queryStoreByPage(Long count, StoreSearchDTO storeSearchDTO, Integer toEnterpriseId, Integer enterpriseId, String key){
        if(StringUtils.isBlank(key)){
            key = ToolUtil.randomUUID();
        }
        int pages = count/pageSize + count%pageSize>0 ? 1 : 0;
        Set<Integer> regionSet = new HashSet<>();
        Set<String> storeTypeSet = new HashSet<>();
        Set<String> storeStatusSet = new HashSet<>();
        List<Integer> existStoreInfoIdList = this.storeService.listStoreIdByFromEnterpriseId(enterpriseId);
        Set<Integer> storeInfoIdSet = new HashSet<>(existStoreInfoIdList);
        int total = 0;
        for(int page = 1; page <= pages; page++){
            ServiceResponse<Page<StoreDTO>> pageServiceResponse = this.storeApiService.listStore(storeSearchDTO, page, pageSize);
            if(pageServiceResponse.isSuccess()){
                Page<StoreDTO> storePage = pageServiceResponse.getResult();
                List<StoreDTO> list = storePage.getResult();
                int isEnd = 0;
                if(CollectionUtils.isNotEmpty(list)){
                    for(int k=0; k< list.size(); k++){
                        StoreDTO storeDTO = list.get(k);
                        regionSet.add(storeDTO.getRegionId());
                        storeTypeSet.add(storeDTO.getStoreType() == null ? null : storeDTO.getStoreType().toString());
                        storeStatusSet.add(storeDTO.getErpStatus() == null ? null : storeDTO.getErpStatus().toString());
                        if(page == pages && k == list.size()-1) {
                            isEnd = 1;
                        }
                        total++;
                        this.sendDataToMq(storeDTO, toEnterpriseId, isEnd, key, total);
                        if(storeInfoIdSet.contains(storeDTO.getStoreInfoId())){
                            storeInfoIdSet.remove(storeDTO.getStoreInfoId());
                        }
                    }
                }
            }
        }
        this.authRegionStatusStoreType(regionSet, storeStatusSet, storeTypeSet, storeInfoIdSet, enterpriseId, toEnterpriseId);
    }

    private void sendDataToMq(StoreDTO storeDTO, Integer toEnterpriseId, int isEnd, String key, int total){
        Map<String, Object> params = new HashMap<>();
        params.put("enterpriseId", storeDTO.getEnterpriseId());
        params.put("store", storeDTO);
        params.put("toEnterpriseId", toEnterpriseId);
        params.put("isEnd", isEnd);
        params.put("key", key);
        params.put("total", total);
        GicMQClient client = GICMQClientUtil.getClientInstance();
        try {
            client.sendMessage("storeAuthorizationMq", JSON.toJSONString(params));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void authRegionStatusStoreType(Set<Integer> regionSet, Set<String> statusSet, Set<String> storeTypeSet,
                                           Set<Integer> storeInfoIdSet, Integer enterpriseId, Integer toEnterpriseId){
        this.storeRegionApiService.authorizeRegionToOther(enterpriseId, toEnterpriseId, new ArrayList<>(regionSet));
        this.storeDictApiService.saveStoreStatusAndStoreType(enterpriseId, statusSet.toArray(new String[0]), storeTypeSet.toArray(new String[0]));
        if(CollectionUtils.isNotEmpty(storeInfoIdSet)){
            this.storeService.cancelAuthStore(new ArrayList<>(storeInfoIdSet), toEnterpriseId);
        }
    }
}
