package com.gic.store.service.outer.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Stack;
import java.util.concurrent.TimeUnit;

import com.alibaba.fastjson.JSON;
import com.gic.store.service.StoreApiService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.error.ErrorCode;
import com.gic.redis.data.util.RedisUtil;
import com.gic.store.constant.StoreGroupConstant;
import com.gic.store.dto.StoreGroupDTO;
import com.gic.store.entity.TabStoreGroup;
import com.gic.store.service.StoreGroupApiService;
import com.gic.store.service.StoreGroupService;
import com.gic.store.service.StoreService;

/**
 * @author guojx
 * @date 2019/6/26 2:24 PM
 */
@Service("storeGroupApiService")
public class StoreGroupApiServiceImpl implements StoreGroupApiService {

    private static final Logger logger = LoggerFactory.getLogger(StoreGroupApiServiceImpl.class);

    private static final int SUCCESS = 1;

    @Autowired
    private StoreService storeService;
    @Autowired
    private StoreGroupService storeGroupService;
    @Autowired
    private StoreApiService storeApiService;

    @Override
    public ServiceResponse<Integer> save(StoreGroupDTO storeGroupDTO) {
        if (storeGroupService.isRepeatName(storeGroupDTO.getStoreGroupName(), null, storeGroupDTO.getEnterpriseId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "名称不能重复");
        }
        int groupLevel = storeGroupDTO.getGroupLevel().intValue();
        int enterpriseId = storeGroupDTO.getEnterpriseId().intValue();
        //父级ID
        ServiceResponse<Integer> parentIdWhenNew = getParentIdWhenNew(groupLevel, enterpriseId, storeGroupDTO.getParentStoreGroupId());
        if (parentIdWhenNew.isSuccess()) {
            storeGroupDTO.setParentStoreGroupId(parentIdWhenNew.getResult());
        } else {
            return parentIdWhenNew;
        }
        storeGroupDTO.setCreateTime(new Date());
        storeGroupDTO.setUpdateTime(new Date());
        storeGroupDTO.setStatus(1);
        if (storeGroupDTO.getIsDefault() == null) {
            storeGroupDTO.setIsDefault(0);
        }
        //排序值
        storeGroupDTO.setSort(selectStoreGroupMaxSort(enterpriseId, groupLevel) + 1);

        int storeGroupId = storeGroupService.insert(storeGroupDTO);

        //update 分组链
        TabStoreGroup storeGroupChain = new TabStoreGroup();
        storeGroupChain.setUpdateTime(new Date());
        storeGroupChain.setStoreGroupId(storeGroupId);
        storeGroupChain.setStoreGroupChain(getStoreGroupChain(storeGroupId));
        storeGroupService.update(storeGroupChain);
        boolean needTransferStore = isNeedTransferStore(storeGroupDTO.getParentStoreGroupId());
        logger.info("needTransferStore:{}", needTransferStore);
        if (needTransferStore) {
            //转移门店
            storeService.updateGroupId(storeGroupDTO.getParentStoreGroupId(), storeGroupId);
        }
        //刷新缓存
        refreshRedisCache(storeGroupId);

        if (groupLevel == StoreGroupConstant.ALL_STORE_LEVEL) {
            //初始化品牌，新增所有门店分组之后，新增一条名称叫做 未分组门店 的默认分组
            StoreGroupDTO noStoreGroup = new StoreGroupDTO();
            noStoreGroup.setEnterpriseId(storeGroupDTO.getEnterpriseId());
            noStoreGroup.setGroupLevel(StoreGroupConstant.FIRST_STORE_GROUP_LEVEL);
            noStoreGroup.setStoreGroupName("未分组门店");
            noStoreGroup.setIsDefault(1);
            save(noStoreGroup);
        }
        return ServiceResponse.success(storeGroupId);
    }

    @Override
    public ServiceResponse<String> update(StoreGroupDTO storeGroupDTO) {
        if (storeGroupDTO.getStoreGroupId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键不能为空");
        }
        int storeGroupId = storeGroupDTO.getStoreGroupId();
        TabStoreGroup oldStoreGroup;
        //初始化的门店分组不允许编辑
        ServiceResponse<TabStoreGroup> oldStoreGroupResponse = validIsDefault(storeGroupId);
        if (oldStoreGroupResponse.isSuccess()) {
            oldStoreGroup = oldStoreGroupResponse.getResult();
        } else {
            return ServiceResponse.failure(oldStoreGroupResponse.getCode(), oldStoreGroupResponse.getMessage());
        }

        int oldGroupLevel = oldStoreGroup.getGroupLevel();
        if (oldGroupLevel != StoreGroupConstant.FIRST_STORE_GROUP_LEVEL) {
            if (storeGroupDTO.getParentStoreGroupId() == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "父级ID参数错误");
            }
            int storeGroupParentId = storeGroupDTO.getParentStoreGroupId();
            //初始化的门店分组下面不允许有子分组
            ServiceResponse<TabStoreGroup> authValid = validIsDefault(storeGroupParentId);
            if (!authValid.isSuccess()) {
                return ServiceResponse.failure(authValid.getCode(), authValid.getMessage());
            }
            //验证上级分组是改动
            int oldStoreGroupParentId = oldStoreGroup.getParentStoreGroupId();
            boolean hasEditParentId = storeGroupParentId != oldStoreGroupParentId;

            if (hasEditParentId) {
                //验证父级ID的层级和当前层级是否相差一，如果是，则正确，否则，数据异常
                TabStoreGroup parentStoreGroup = storeGroupService.getStoreGroupById(storeGroupParentId);
                if (parentStoreGroup == null) {
                    return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "父级ID数据错误，查询不到数据");
                }
                if (oldGroupLevel - parentStoreGroup.getGroupLevel() != 1) {
                    return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "父级ID数据错误，不是当前的上级层级");
                }
                //如果修改了上级分组，则该分组下的所有子级分组都需要修改分组链数据
                storeGroupService.updateStoreGroupChainByStoreGroupId(storeGroupId, oldStoreGroupParentId,
                        storeGroupParentId);

                //修改父级ID
                TabStoreGroup tabStoreGroup = new TabStoreGroup();
                tabStoreGroup.setStoreGroupId(storeGroupDTO.getStoreGroupId());
                tabStoreGroup.setParentStoreGroupId(storeGroupParentId);
                tabStoreGroup.setUpdateTime(new Date());
                storeGroupService.update(tabStoreGroup);

                //如果上级分组有门店，则新建一个未分组数据，然后把所有门店放进去
                if (isNeedTransferStore(storeGroupParentId)) {
                    StoreGroupDTO noStoreGroup = new StoreGroupDTO();
                    noStoreGroup.setEnterpriseId(storeGroupDTO.getEnterpriseId());
                    noStoreGroup.setGroupLevel(oldStoreGroup.getGroupLevel());
                    noStoreGroup.setStoreGroupName("未分组门店" + System.currentTimeMillis());
                    noStoreGroup.setParentStoreGroupId(storeGroupParentId);
                    ServiceResponse<Integer> noStoreGroupId = save(noStoreGroup);
                    //转移门店
                    storeService.updateGroupId(storeGroupParentId, noStoreGroupId.getResult());
                }

                //刷新缓存
                refreshRedisCache(storeGroupId);
                List<StoreGroupDTO> listChidrend = this.listStoreGroupAndChildren(storeGroupParentId).getResult();
                logger.info("当前节点下的所有子节点", JSON.toJSONString(listChidrend));
                for(StoreGroupDTO dto : listChidrend){
                    this.storeApiService.refreshStoreIndex(storeGroupDTO.getEnterpriseId(), dto.getStoreGroupId());
                }
            }
        } else {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "一级分组不需要修改上级分组");
        }
        return ServiceResponse.success(oldStoreGroup.getStoreGroupName());
    }

    @Override
    public ServiceResponse<Boolean> tipWhenUpdateParentId(Integer parentStoreGroupId) {
        return ServiceResponse.success(isNeedTransferStore(parentStoreGroupId));
    }

    @Override
    public ServiceResponse<Boolean> tipWhenSaveStoreGroup(Integer groupLevel, Integer enterpriseId, Integer parentStoreGroupId) {
        //父级ID
        ServiceResponse<Integer> parentIdWhenNew = getParentIdWhenNew(groupLevel, enterpriseId, parentStoreGroupId);
        if (parentIdWhenNew.isSuccess()) {
            parentStoreGroupId = parentIdWhenNew.getResult();
        } else {
            return ServiceResponse.failure(parentIdWhenNew.getCode(), parentIdWhenNew.getMessage());
        }
        return ServiceResponse.success(isNeedTransferStore(parentStoreGroupId));
    }

    @Override
    public ServiceResponse<Integer> updateStoreGroupName(StoreGroupDTO storeGroupDTO) {
        if (storeGroupDTO.getStoreGroupId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键不能为空");
        }
        if (StringUtils.isBlank(storeGroupDTO.getStoreGroupName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "名称不能为空");
        }
        ServiceResponse<TabStoreGroup> authValid = validIsDefault(storeGroupDTO.getStoreGroupId());
        if (!authValid.isSuccess()) {
            return ServiceResponse.failure(authValid.getCode(), authValid.getMessage());
        }
        if (storeGroupService.isRepeatName(storeGroupDTO.getStoreGroupName(), storeGroupDTO.getStoreGroupId(),
                storeGroupDTO.getEnterpriseId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "名称不能重复");
        }
        //修改名称
        TabStoreGroup tabStoreGroup = new TabStoreGroup();
        tabStoreGroup.setStoreGroupName(storeGroupDTO.getStoreGroupName());
        tabStoreGroup.setStoreGroupId(storeGroupDTO.getStoreGroupId());
        tabStoreGroup.setUpdateTime(new Date());
        storeGroupService.update(tabStoreGroup);

        //刷新缓存
        refreshRedisCache(storeGroupDTO.getStoreGroupId());
        this.storeApiService.refreshStoreIndex(storeGroupDTO.getEnterpriseId(), storeGroupDTO.getStoreGroupId());
        return ServiceResponse.success(SUCCESS);
    }

    @Override
    public ServiceResponse<String> sort(StoreGroupDTO storeGroupDTO, boolean isUp) {
        TabStoreGroup tabStoreGroup = storeGroupService.getStoreGroupById(storeGroupDTO.getStoreGroupId());
        if (tabStoreGroup == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店分组主键有误");
        }
        int groupLevel = tabStoreGroup.getGroupLevel();
        int enterpriseId = tabStoreGroup.getEnterpriseId();
        int sort = tabStoreGroup.getSort();

        TabStoreGroup needUpdateSortStoreGroup;
        if (isUp) {
            needUpdateSortStoreGroup = storeGroupService.getStoreGroupWhenMaxSort(enterpriseId, groupLevel, sort);
            if (needUpdateSortStoreGroup == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已经是第一条，不能上移");
            }
        } else {
            needUpdateSortStoreGroup = storeGroupService.getStoreGroupWhenMinSort(enterpriseId, groupLevel, sort);
            if (needUpdateSortStoreGroup == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已经是最后一条，不能下移");
            }
        }
        updateSort(needUpdateSortStoreGroup.getStoreGroupId(), sort);
        updateSort(storeGroupDTO.getStoreGroupId(), needUpdateSortStoreGroup.getSort());
        return ServiceResponse.success(tabStoreGroup.getStoreGroupName());
    }

    @Override
    public ServiceResponse<List<StoreGroupDTO>> listStoreGroup(StoreGroupDTO storeGroupDTO) {
        List<TabStoreGroup> tabStoreGroupList = new ArrayList<>(16);
        if (storeGroupDTO.getStoreGroupId() != null) {
            //如果有ID，则查询出该分组以及子集分组数据
            tabStoreGroupList.addAll(storeGroupService.listStoreGroupAndChildren(storeGroupDTO.getStoreGroupId()));

            //获取父级数据
            TabStoreGroup tabStoreGroup = storeGroupService.getStoreGroupById(storeGroupDTO.getStoreGroupId());
            Stack<TabStoreGroup> parentStoreGroupList = new Stack<>();
            listParentStoreGroup(tabStoreGroup.getParentStoreGroupId(), parentStoreGroupList);
            while (!parentStoreGroupList.empty()) {
                tabStoreGroupList.add(parentStoreGroupList.pop());
            }
        }

        if (storeGroupDTO.getGroupLevel() != null) {
            tabStoreGroupList.addAll(storeGroupService.listStoreGroup(storeGroupDTO.getGroupLevel(),
                    storeGroupDTO.getEnterpriseId(), storeGroupDTO.getStoreGroupName()));
        } else {
            //查询全部
            tabStoreGroupList.addAll(storeGroupService.listStoreGroup(null, storeGroupDTO.getEnterpriseId(), null));
        }

        return ServiceResponse.success(EntityUtil.changeEntityListNew(StoreGroupDTO.class, tabStoreGroupList));
    }

    @Override
    public ServiceResponse<String> remove(int storeGroupId) {
        TabStoreGroup record = storeGroupService.getStoreGroupById(storeGroupId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店分组主键有误");
        }
        if (isContainStore(storeGroupId)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "该分组下存在门店，不能删除");
        }
        ServiceResponse<TabStoreGroup> authValid = validIsDefault(storeGroupId);
        if (!authValid.isSuccess()) {
            return ServiceResponse.failure(authValid.getCode(), authValid.getMessage());
        }
        TabStoreGroup tabStoreGroup = new TabStoreGroup();
        tabStoreGroup.setStoreGroupId(storeGroupId);
        tabStoreGroup.setStatus(0);
        storeGroupService.update(tabStoreGroup);

        refreshRedisCache(storeGroupId);
        return ServiceResponse.success(record.getStoreGroupName());
    }

    @Override
    public ServiceResponse<Integer> insertDefaultStoreGroup(int enterpriseId) {
        StoreGroupDTO noStoreGroup = new StoreGroupDTO();
        noStoreGroup.setEnterpriseId(enterpriseId);
        noStoreGroup.setGroupLevel(StoreGroupConstant.ALL_STORE_LEVEL);
        noStoreGroup.setStoreGroupName("所有门店");
        noStoreGroup.setIsDefault(StoreGroupConstant.IS_DEFAULT);
        return save(noStoreGroup);
    }

    @Override
    public ServiceResponse<String> dragSort(Integer storeGroupId, Integer setSortValue) {
        TabStoreGroup tabStoreGroup = storeGroupService.getStoreGroupById(storeGroupId);
        if (tabStoreGroup == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键错误，查询不到数据");
        }
        
        if (setSortValue == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "拖拽排序值不能为空");
        }
        storeGroupService.dragSort(tabStoreGroup, setSortValue);
        return ServiceResponse.success(tabStoreGroup.getStoreGroupName());
    }

    @Override
    public ServiceResponse<List<StoreGroupDTO>> listStoreGroupAndChildren(Integer storeGroupId) {
        String key = "enterprise:store:storeGroup:list:" + storeGroupId;
        List<StoreGroupDTO> resultList = (List<StoreGroupDTO>) RedisUtil.getCache(key);
        if (resultList == null) {
            resultList = EntityUtil.changeEntityListNew(StoreGroupDTO.class,
                    storeGroupService.listStoreGroupAndChildren(storeGroupId));
            RedisUtil.setCache(key, resultList, 6 * 30L, TimeUnit.DAYS);
        }
        return ServiceResponse.success(resultList);
    }

    @Override
    public ServiceResponse<StoreGroupDTO> getStoreGroupById(Integer storeGroupId) {
        String key = "enterprise:store:storeGroup:" + storeGroupId;
        StoreGroupDTO result = (StoreGroupDTO) RedisUtil.getCache(key);
        if (result == null) {
            result = EntityUtil.changeEntityNew(StoreGroupDTO.class, storeGroupService.getStoreGroupById(storeGroupId));
            RedisUtil.setCache(key, result, 6 * 30L, TimeUnit.DAYS);
        }
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<Boolean> isLeafNode(Integer storeGroupId) {
        TabStoreGroup tabStoreGroup = storeGroupService.getStoreGroupById(storeGroupId);
        if (tabStoreGroup == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键错误，查询不到数据");
        }
        return ServiceResponse.success(storeGroupService.isLeafNode(storeGroupId));
    }

    @Override
    public ServiceResponse<List<StoreGroupDTO>> listStoreGroupByIds(List<Integer> storeGroupIdList, Integer enterpriseId) {
        List<TabStoreGroup> list = this.storeGroupService.listStoreGroupByIds(storeGroupIdList, enterpriseId);
        return ServiceResponse.success(EntityUtil.changeEntityListByJSON(StoreGroupDTO.class, list));
    }

    /**
     * 刷新缓存，删除缓存
     * @param storeGroupId
     */
    private void refreshRedisCache(Integer storeGroupId) {
        RedisUtil.delCache("enterprise:store:storeGroup:" + storeGroupId);
        RedisUtil.delCache("enterprise:store:storeGroup:list:" + storeGroupId);
    }

    private int selectStoreGroupMaxSort(Integer enterpriseId, Integer groupLevel) {
        return storeGroupService.selectStoreGroupMaxSort(enterpriseId, groupLevel);
    }

    private String getStoreGroupChain(Integer storeGroupId) {
        TabStoreGroup tabStoreGroup = storeGroupService.getStoreGroupById(storeGroupId);
        int parentId = tabStoreGroup.getParentStoreGroupId();
        if (parentId != StoreGroupConstant.ALL_STORE_LEVEL) {
            //如果不是0，说明不是所有门店
            TabStoreGroup parentStoreGroup = storeGroupService.getStoreGroupById(parentId);
            return parentStoreGroup.getStoreGroupChain() + storeGroupId + "_";
        } else {
            return "_0_" + storeGroupId + "_";
        }
    }

    private void updateSort(int storeGroupId, int sort) {
        TabStoreGroup tabStoreGroup = new TabStoreGroup();
        tabStoreGroup.setStoreGroupId(storeGroupId);
        tabStoreGroup.setSort(sort);
        storeGroupService.update(tabStoreGroup);
    }

    /**
     * 是否需要转移门店，新增门店，判断上级门店是否是子节点，并且是否有门店
     * @param parentStoreGroupId
     * @return
     */
    private boolean isNeedTransferStore(Integer parentStoreGroupId) {
        if (storeGroupService.isLeafNode(parentStoreGroupId)
                && isContainStore(parentStoreGroupId)) {
            return true;
        } else {
            return false;
        }
    }

    private boolean isContainStore(int storeGroupId) {
        Integer result = storeService.countByGroupId(storeGroupId);
        if (result == null || result < 1) {
            return false;
        }
        return true;
    }

    private void listParentStoreGroup(int parentId, Stack<TabStoreGroup> parentStoreGroupList) {
        TabStoreGroup tabStoreGroup = storeGroupService.getStoreGroupById(parentId);
        if (tabStoreGroup != null) {
            parentStoreGroupList.push(tabStoreGroup);
            if (tabStoreGroup.getParentStoreGroupId() != StoreGroupConstant.ALL_STORE_LEVEL) {
                listParentStoreGroup(tabStoreGroup.getParentStoreGroupId(), parentStoreGroupList);
            }
        }
    }

    /**
     * 如果是默认门店，不可编辑
     * @param storeGroupId
     */
    private ServiceResponse<TabStoreGroup> validIsDefault(int storeGroupId) {
        TabStoreGroup storeGroup = storeGroupService.getStoreGroupById(storeGroupId);
        if (storeGroup == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店分组主键错误，数据未查到");
        }
        boolean isDefault = storeGroup.getIsDefault() == StoreGroupConstant.IS_DEFAULT;
        if (isDefault) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "默认分组：未分组门店，不可编辑");
        }
        return ServiceResponse.success(storeGroup);
    }

    /**
     * 新增的时候获取父级ID
     * @param groupLevel
     * @param enterpriseId
     * @param parentId
     * @return
     */
    private ServiceResponse<Integer> getParentIdWhenNew(Integer groupLevel, Integer enterpriseId, Integer parentId) {
        //最小是0级，即所有门店
        if (groupLevel < StoreGroupConstant.ALL_STORE_LEVEL) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分组层级参数数据错误");
        }
        //最大是6级
        if (groupLevel > StoreGroupConstant.LAST_STORE_GROUP_LEVEL) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分组层级参数数据错误");
        }
        //所有门店的父级ID为0
        if (groupLevel == StoreGroupConstant.ALL_STORE_LEVEL) {
            TabStoreGroup allStoreGroup = storeGroupService.selectAllStoreGroup(enterpriseId);
            if (allStoreGroup != null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "所有门店已存在");
            }
            return ServiceResponse.success(StoreGroupConstant.ALL_STORE_LEVEL);
        } else if (groupLevel == StoreGroupConstant.FIRST_STORE_GROUP_LEVEL) {
            //如果是第一级，则父级ID取所有门店，所有门店在初始化品牌的时候创建
            TabStoreGroup allStoreGroup = storeGroupService.selectAllStoreGroup(enterpriseId);
            return ServiceResponse.success(allStoreGroup.getStoreGroupId());
        } else {
            if (parentId == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "父级ID参数错误");
            }
            TabStoreGroup tabStoreGroup = storeGroupService.getStoreGroupById(parentId);
            if (tabStoreGroup == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "父级ID参数错误");
            }
            if (groupLevel - tabStoreGroup.getGroupLevel() != 1) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "父级ID参数错误，不是" + (groupLevel - 1) + "级");
            }
            ServiceResponse<TabStoreGroup> validIsDefault = validIsDefault(parentId);
            if (!validIsDefault.isSuccess()) {
                return ServiceResponse.failure(validIsDefault.getCode(), validIsDefault.getMessage());
            }
            return ServiceResponse.success(parentId);
        }
    }
}
