package com.gic.store.service.outer.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.store.dto.StoreStatusSettingDTO;
import com.gic.store.entity.TabStoreStatusSetting;
import com.gic.store.service.StoreRegionRelService;
import com.gic.store.service.StoreStatusSettingApiService;
import com.gic.store.service.StoreStatusSettingService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service("storeStatusSettingApiService")
public class StoreStatusSettingApiServiceImpl implements StoreStatusSettingApiService {
    @Autowired
    private StoreStatusSettingService storeStatusSettingService;
    @Autowired
    private StoreRegionRelService storeRegionRelService;

    @Override
    public ServiceResponse saveSetting(StoreStatusSettingDTO dto) {
        boolean owner = storeRegionRelService.validIsOwn(dto.getEnterpriseId(), dto.getRegionId());
        if (owner) {
            boolean result = this.storeStatusSettingService.saveSetting(EntityUtil.changeEntityByJSON(TabStoreStatusSetting.class, dto));
            if(result){
                return ServiceResponse.success();
            }
        }
        return EnterpriseServiceResponse.failure(ErrorCode.OPERATION_FAILED);
    }

    @Override
    public ServiceResponse<List<StoreStatusSettingDTO>> listSetting(Integer enterpriseId) {
        List<TabStoreStatusSetting> list = this.storeStatusSettingService.listTabStoreStatusSetting(enterpriseId);
        return ServiceResponse.success(EntityUtil.changeEntityListByJSON(StoreStatusSettingDTO.class, list));
    }

    @Override
    public ServiceResponse delete(Integer enterpriseId, Integer id) {
        Boolean result = this.storeStatusSettingService.delete(enterpriseId, id);
        if(result){
            return ServiceResponse.success();
        }
        return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
    }
}
