package com.gic.store.service.outer.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.error.ErrorCode;
import com.gic.store.dto.StoreTagDTO;
import com.gic.store.entity.TabStoreTag;
import com.gic.store.service.StoreTagApiService;
import com.gic.store.service.StoreTagService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service("storeTagApiService")
public class StoreTagApiServiceImpl implements StoreTagApiService{
    @Autowired
    private StoreTagService storeTagService;

    @Override
    public ServiceResponse<Integer> save(StoreTagDTO dto) {
        //param valid
        if (dto.getEnterpriseId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID不能为空");
        }
        if (dto.getStoreWidgetId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店选择器未选择");
        }
        if (StringUtils.isBlank(dto.getStoreTagName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "标签名称不能为空");
        }
        return ServiceResponse.success(storeTagService.save(dto));
    }

    @Override
    public ServiceResponse<Integer> update(StoreTagDTO dto) {
        //param valid
        TabStoreTag record = storeTagService.getById(dto.getStoreTagId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "标签ID非法，查无数据");
        }
        if (dto.getStoreWidgetId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店选择器未选择");
        }
        if (StringUtils.isBlank(dto.getStoreTagName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "标签名称不能为空");
        }
        //set value
        dto.setEnterpriseId(record.getEnterpriseId());
        dto.setCreateTime(record.getCreateTime());
        dto.setStatus(record.getStatus());

        storeTagService.update(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<String> delete(Integer storeTagId) {
        //param valid
        TabStoreTag record = storeTagService.getById(storeTagId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "标签ID非法，查无数据");
        }

        StoreTagDTO dto = new StoreTagDTO();
        dto.setStoreTagId(storeTagId);
        dto.setStatus(0);

        storeTagService.update(dto);
        return ServiceResponse.success(record.getStoreTagName());
    }

    @Override
    public ServiceResponse<StoreTagDTO> getByStoreTagId(Integer storeTagId) {
        TabStoreTag record = storeTagService.getById(storeTagId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "标签ID非法，查无数据");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(StoreTagDTO.class, record));
    }

    @Override
    public ServiceResponse<List<StoreTagDTO>> listByEnterprise(Integer enterpriseId) {
        List<TabStoreTag> list = storeTagService.listByEnterprise(enterpriseId);
        return ServiceResponse.success(EntityUtil.changeEntityListNew(StoreTagDTO.class, list));
    }
}
