package com.gic.store.strategy.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.commons.util.GlobalInfo;
import com.gic.store.constant.WeekdayEnum;
import com.gic.store.service.StoreLogService;
import com.gic.store.service.StoreService;
import com.gic.store.strategy.BulkUpdateStoreStrtegy;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Component("3")
public class StoreBusinessTimeStrategyImpl implements BulkUpdateStoreStrtegy {

    @Autowired
    private StoreLogService storeLogService;

    @Override
    public int bulkUpdateStore(String storeInfoIds, Integer enterpriseId, String value, StoreService storeService, Integer operatorId, String operatorName) {
        int i = storeService.bulkUpdateStoreBusinessTime(storeInfoIds, enterpriseId, value);
        if (i > 0) {
            List<Integer> storeInfoIdList;
            if ("all".equals(storeInfoIds)) {
                storeInfoIdList = storeService.listAllStoreId(enterpriseId);
            } else {
                storeInfoIdList = new ArrayList<>();
                String[] storeInfoIdArr = storeInfoIds.split(",");
                for (String s : storeInfoIdArr) {
                    storeInfoIdList.add(Integer.valueOf(s));
                }
            }
            //            [{"weekday":"1,2","openTime":"09:42","closeTime":"09:42"}]
            StringBuilder btSb = new StringBuilder("营业时间变更为【  ");
            List<JSONObject> businessTimeList = JSON.parseArray(value, JSONObject.class);
            for (JSONObject storeBusinessTime : businessTimeList) {
                String weekday = storeBusinessTime.getString("weekday");
                List<Integer> list = Stream.of(weekday.split(GlobalInfo.FLAG_COMMA)).filter(e -> StringUtils.isNotBlank(e)).map(Integer::parseInt).collect(Collectors.toList());
                if (list.size() >= 3 && list.get(list.size() - 1) - list.get(0) == list.size() - 1) {
                    // 超过三条的需要判断
                    // 都是连续的
                    btSb.append(WeekdayEnum.parse(list.get(0)) + "至" + WeekdayEnum.parse(list.get(list.size() - 1)));
                } else {
                    // 直接解析返回
                    StringBuilder sb = new StringBuilder();
                    for (Integer code : list) {
                        sb.append(WeekdayEnum.parse(code)).append("、");
                    }
                    sb.deleteCharAt(sb.length() - 1);
                    btSb.append(sb);
                }
                btSb.append(":").append(storeBusinessTime.getString("openTime")).append("~").append(storeBusinessTime.getString("endTime")).append(",");
            }
            btSb.deleteCharAt(btSb.length() - 1);
            btSb.append("】");
            storeLogService.saveBulkStoreInfoLog(storeInfoIdList, enterpriseId, operatorId, operatorName, btSb.toString());
        }
        return i;
    }
}
