package com.gic.store.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.error.ErrorCode;
import com.gic.store.constant.StoreFieldTypeEnum;
import org.apache.commons.lang3.StringUtils;

/**
 * @author guojx
 * @date 2019/7/1 9:55 AM
 */
public class StoreFieldValidParamUtil {
    /**
     * 文本类型
     */
    public final static String TEXT_TYPE = "textType";
    /**
     * 小数点位数
     */
    public final static String FLOAT_DIGITS = "floatDigits";
    /**
     * 时间精度
     */
    public final static String DATE_TYPe = "dateType";
    /**
     * 最大位数
     */
    public final static String MAX_DIGITS = "maxDigits";
    public final static String MIN_DIGITS = "minDigits";
    /**
     * 最大长度
     */
    public final static String MAX_LENGTH = "maxLength";
    public final static String MIN_LENGTH = "minLength";
    /**
     * 提示信息
     */
    public final static String TIP_MESSAEG = "tipMessage";
    /**
     * 是否必须
     */
    public final static String IS_MUST = "isMust";
    /**
     * 允许重复
     */
    public final static String ALLOW_REPEAT = "allowRepeat";
    /**
     * 允许修改
     */
    public final static String ALLOW_EDIT = "allowEdit";

    public final static String FIELD_NAME = "storeFieldDetail";

    public final static String ERROR_MESSAGE = FIELD_NAME + "参数值错误";

    public static ServiceResponse validStoreFieldDetail(Integer fieldType, String json) {
        StoreFieldTypeEnum typeEnum = StoreFieldTypeEnum.getByCode(fieldType);
        switch (typeEnum) {
            case TEXT:
                return validTextTypeParam(json);
            case DATE:
                return validDateTypeParam(json);
            case CHECK:
                return validCheckTypeParam(json);
            case RADIO:
                return validRadioTypeParam(json);
            case REAL_NUMBER:
                return validRealNumberTypeParam(json);
            default:
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "字段类型参数值错误");
        }
    }

    public static ServiceResponse validTextTypeParam(String json) {
        JSONObject fieldDetail = (JSONObject) JSON.parse(json);
        if (fieldDetail.containsKey(TEXT_TYPE)) {
            String textType = fieldDetail.getString(TEXT_TYPE);
            if (StringUtils.isBlank(textType)) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), ERROR_MESSAGE);
            }
        } else {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), ERROR_MESSAGE);
        }
        if (!fieldDetail.containsKey(MAX_LENGTH)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), ERROR_MESSAGE);
        }
        if (!fieldDetail.containsKey(MIN_LENGTH)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), ERROR_MESSAGE);
        }
        return validCommon(fieldDetail);
    }

    public static ServiceResponse validRadioTypeParam(String json) {
        JSONObject fieldDetail = (JSONObject) JSON.parse(json);
        return validCommon(fieldDetail);
    }


    public static ServiceResponse validCheckTypeParam(String json) {
        JSONObject fieldDetail = (JSONObject) JSON.parse(json);
        return validCommon(fieldDetail);
    }


    public static ServiceResponse validRealNumberTypeParam(String json) {
        JSONObject fieldDetail = (JSONObject) JSON.parse(json);
        if (fieldDetail.containsKey(FLOAT_DIGITS)) {
            Integer floatDigits = fieldDetail.getInteger(FLOAT_DIGITS);
            if (floatDigits == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), ERROR_MESSAGE);
            }
        } else {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), ERROR_MESSAGE);
        }
        if (!fieldDetail.containsKey(MAX_DIGITS)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), ERROR_MESSAGE);
        }
        if (!fieldDetail.containsKey(MIN_DIGITS)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), ERROR_MESSAGE);
        }
        return validCommon(fieldDetail);
    }

    public static ServiceResponse validDateTypeParam(String json) {
        JSONObject fieldDetail = (JSONObject) JSON.parse(json);
        if (fieldDetail.containsKey(DATE_TYPe)) {
            String floatDigits = fieldDetail.getString(DATE_TYPe);
            if (StringUtils.isBlank(floatDigits)) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), ERROR_MESSAGE);
            }
        } else {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), ERROR_MESSAGE);
        }
        return validCommon(fieldDetail);
    }

    private static ServiceResponse validCommon(JSONObject fieldDetail) {
        if (!fieldDetail.containsKey(TIP_MESSAEG)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), ERROR_MESSAGE);
        }
        if (!fieldDetail.containsKey(IS_MUST)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), ERROR_MESSAGE);
        }
        if (!fieldDetail.containsKey(ALLOW_EDIT)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), ERROR_MESSAGE);
        }
        if (!fieldDetail.containsKey(ALLOW_REPEAT)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), ERROR_MESSAGE);
        }
        return ServiceResponse.success();
    }
}
