package com.gic.store.web.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.constants.QrcodeTypeEnum;
import com.gic.download.qo.QrcodeContent;
import com.gic.download.utils.*;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.context.RequestContext;
import com.gic.enterprise.dto.EnterpriseLicenseDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.service.DownloadReportApiService;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.enterprise.service.QrCodeApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.log.api.service.LogApiService;
import com.gic.member.api.dto.StoreBatchUpdateDTO;
import com.gic.member.api.service.StoreUpdateApiService;
import com.gic.redis.data.util.RedisUtil;
import com.gic.store.constant.*;
import com.gic.store.dto.*;
import com.gic.store.service.*;
import com.gic.store.utils.StoreRedisKeyUtils;
import com.gic.store.web.qo.PageQO;
import com.gic.store.web.qo.store.QrcodeQO;
import com.gic.store.web.qo.store.StoreExportQO;
import com.gic.store.web.qo.store.StoreQO;
import com.gic.store.web.vo.StoreExportFieldVO;
import com.gic.store.web.vo.StoreListSourceVO;
import com.gic.store.web.vo.StoreOverflowVO;
import com.gic.wechat.business.api.service.fwh.WeixinBaseFunService;
import com.gic.wechat.business.api.service.wxa.WeixinWxaFunService;
import com.google.common.base.Joiner;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

/**
 * @author zhiwj
 * @date 2019/6/28
 */
@RestController
public class StoreController extends DownloadUtils {

    private static final Logger logger = LogManager.getLogger(StoreController.class);

    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private StoreDictApiService storeDictApiService;
    @Autowired
    private LogApiService logApiService;
    @Autowired
    private StoreLogApiService storeLogApiService;
    @Autowired
    private StoreFieldApiService storeFieldApiService;
    @Autowired
    private StoreFieldSelectApiService storeFieldSelectApiService;
    @Autowired
    private QrCodeApiService qrCodeApiService;
    @Autowired
    private WeixinBaseFunService weixinBaseFunService;
    @Autowired
    private WeixinWxaFunService weixinWxaFunService;
    @Autowired
    private DownloadReportApiService downloadReportApiService;
    @Autowired
    private StoreTagApiService storeTagApiService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;
    @Autowired
    private StoreUpdateApiService storeUpdateApiService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;

    private Map<Integer, SimpleDateFormat> map = new ConcurrentHashMap<>();
    private Map<Integer, SimpleDateFormat> map1 = new ConcurrentHashMap<>();

    @RequestMapping("/save-store")
    public RestResponse saveOrUpdate(@Validated({StoreQO.SaveValidView.class}) StoreQO storeQO) {
        logger.info("门店参数：{}", JSON.toJSONString(storeQO));
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        StoreDTO storeDTO = EntityUtil.changeEntityByOrika(StoreDTO.class, storeQO);
        storeDTO.setEnterpriseId(enterpriseId);
        String photos = storeQO.getPhotos();
        if (StringUtils.isNotBlank(photos)) {
            List<StorePhotoDTO> photoList = JSON.parseArray(photos, StorePhotoDTO.class);
            storeDTO.setPhotoList(photoList);
        }
        String businessTimes = storeQO.getBusinessTimes();
        if (StringUtils.isNotBlank(businessTimes)) {
            List<StoreBusinessTimeDTO> businessTimeList = JSON.parseArray(businessTimes, StoreBusinessTimeDTO.class);
            storeDTO.setBusinessTimeList(businessTimeList);
        }
        storeDTO.setCreateType(CreateTypeEnum.BACKGROUND_ADD.getCode());
        String fieldJson = storeQO.getStoreExtends();
        List<StoreExtendDTO> extendList = JSON.parseArray(fieldJson, StoreExtendDTO.class);
        storeDTO.setStoreExtendList(extendList);
        // 判断是否手动修改过分组
        Integer isEditGroup = getIsEditGroup(enterpriseId, storeDTO);
        logger.info("isEditGroup:{}", isEditGroup);
        storeDTO.setIsEditStoreGroup(isEditGroup);
        storeDTO.setReason(StoreLogReasonEnum.WEB.getCode());
        storeDTO.setRemark(storeQO.getStoreId() == null ? "手工新增" : "单个编辑");
        storeDTO.setOperatorId(UserDetailUtils.getUserDetail().getUserId());
        storeDTO.setOperatorName(UserDetailUtils.getUserDetail().getUserInfo().getUserName());
        ServiceResponse<StoreDTO> serviceResponse = storeApiService.saveOrUpdate(storeDTO);
        if (serviceResponse.isSuccess()) {
            LogUtils.createLog((storeQO.getStoreId() == null ? "新增" : "编辑") + "门店", storeQO.getStoreName());
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/store-detail")
    public RestResponse getStoreDetail(Integer storeId) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<StoreDTO> serviceResponse = storeApiService.getStoreById(enterpriseId, storeId);
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/store-list")
    public RestResponse storeList(@RequestBody StoreSearchDTO storeSearchDTO) {
        logger.info("查询条件：{}", JSON.toJSONString(storeSearchDTO));
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Integer userId = UserDetailUtils.getUserDetail().getUserId();
        storeSearchDTO.setEnterpriseId(enterpriseId);
        if(UserDetailUtils.getUserDetail().getUserResourceInfo() != null){
            storeSearchDTO.setStoreResource(UserDetailUtils.getUserDetail().getUserResourceInfo().getStoreResource());
        }

        //门店标签
        ServiceResponse<StoreTagDTO> storeTagResult = storeTagApiService.getByStoreTagId(storeSearchDTO.getStoreTagId());
        if (storeTagResult.isSuccess()) {
            StoreWidgetDTO storeWidgetDTO = this.storeWidgetApiService.getStoreWidget(Integer.valueOf(storeTagResult.getResult().getStoreWidgetId().toString())).getResult();
            if(storeWidgetDTO != null){
                String authSearchParam = storeWidgetDTO.getAuthSearchParam();
                String searchParam = storeWidgetDTO.getSearchParam();
                if(StringUtils.isNotBlank(authSearchParam) && !"{}".equals(authSearchParam)){
                    storeSearchDTO.setAuthSearchJson(authSearchParam);
                }
                storeSearchDTO.setSearchJson(searchParam);
            }
        }

        String key = StoreRedisKeyUtils.getStoreListSourceKey(enterpriseId, userId);
        Object obj = RedisUtil.getCache(key);
        String returnFileds = StoreESFieldsEnum.STOREID.getField();
        if(obj == null){
            List<StoreListSourceVO> list = this.getStoreAllListSource(enterpriseId);
            for(StoreListSourceVO vo : list){
                if(vo.getChecked().intValue() == 1){
                    returnFileds += "," + vo.getValue();
                }
            }
        }else {
            returnFileds += ","+obj.toString();
        }
        ServiceResponse<Page<StoreDTO>> serviceResponse = storeApiService.listStore(storeSearchDTO, storeSearchDTO.getCurrentPage(), storeSearchDTO.getPageSize(), returnFileds);
        if (serviceResponse.isSuccess()) {
            Page<StoreDTO> page = serviceResponse.getResult();
            if(page != null){
                List<StoreDTO> result = page.getResult();
                if(CollectionUtils.isNotEmpty(result)){
                    this.convertCustomField(result);
                }
                page.setResult(result);
            }
            return RestResponse.success(page);
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("online-store-count")
    public RestResponse onlineStoreCount(@RequestBody StoreSearchDTO storeSearchDTO){
        //门店标签
        ServiceResponse<StoreTagDTO> storeTagResult = storeTagApiService.getByStoreTagId(storeSearchDTO.getStoreTagId());
        if (storeTagResult.isSuccess()) {
            StoreWidgetDTO storeWidgetDTO = this.storeWidgetApiService.getStoreWidget(Integer.valueOf(storeTagResult.getResult().getStoreWidgetId().toString())).getResult();
            if(storeWidgetDTO != null){
                String authSearchParam = storeWidgetDTO.getAuthSearchParam();
                String searchParam = storeWidgetDTO.getSearchParam();
                if(StringUtils.isNotBlank(authSearchParam) && !"{}".equals(authSearchParam)){
                    storeSearchDTO.setAuthSearchJson(authSearchParam);
                }
                storeSearchDTO.setSearchJson(searchParam);
            }
        }
        storeSearchDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        storeSearchDTO.setStatus(StoreStatusEnum.ONLINE.getCode());
        if(UserDetailUtils.getUserDetail().getUserResourceInfo() != null){
            storeSearchDTO.setStoreResource(UserDetailUtils.getUserDetail().getUserResourceInfo().getStoreResource());
        }
        ServiceResponse<Long> response = this.storeApiService.queryStoreCountFromES(storeSearchDTO);
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("store-list-source")
    public RestResponse storeListSource() {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Integer userId = UserDetailUtils.getUserDetail().getUserId();
        List<StoreListSourceVO> list = this.getStoreAllListSource(enterpriseId);
        Object obj = RedisUtil.getCache(StoreRedisKeyUtils.getStoreListSourceKey(enterpriseId, userId));
        if (obj == null) {
            return RestResponse.success(list);
        }
        String[] sourcesArr = obj.toString().split(",");
        for (StoreListSourceVO storeListSourceVO : list) {
            boolean flag = false;
            for (String s : sourcesArr) {
                if (s.equals(storeListSourceVO.getValue())) {
                    storeListSourceVO.setChecked(1);
                    flag = true;
                }
            }
            if(!flag){
                storeListSourceVO.setChecked(0);
            }
        }
        return RestResponse.success(list);
    }

    @RequestMapping("store-list-source-set")
    public RestResponse storeListSourceSet(String sources) {
        if (StringUtils.isBlank(sources)) {
            return EnterpriseRestResponse.failure(ErrorCode.MISS_PARAMETER);
        }
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Integer userId = UserDetailUtils.getUserDetail().getUserId();
        RedisUtil.setCache(StoreRedisKeyUtils.getStoreListSourceKey(enterpriseId, userId), sources);
        logger.info("自定义门店列表字段配置:{}", JSON.toJSONString(RedisUtil.getCache(StoreRedisKeyUtils.getStoreListSourceKey(enterpriseId, userId))));
        return RestResponse.success();
    }

    @RequestMapping("list-store-overflow")
    public RestResponse listStoreOverFlow(String search, PageQO pageQO) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Page<StoreDTO> page = new Page<>(pageQO.getCurrentPage(), pageQO.getPageSize());
        ServiceResponse<Page<StoreDTO>> response = this.storeApiService.listStoreOverFlow(enterpriseId, page, search);
        if (response.isSuccess()) {
            page = response.getResult();
            Page<StoreOverflowVO> page1 = new Page<>();
            page1.setTotalCount(page.getTotalCount());
            page1.setCurrentPage(page.getCurrentPage());
            page1.setPageSize(page.getPageSize());
            page1.setResult(EntityUtil.changeEntityListByJSON(StoreOverflowVO.class, page.getResult()));
            return RestResponse.success(page1);
        }
        return EnterpriseRestResponse.failure(response);
    }

    @RequestMapping("store-license-limit")
    public RestResponse storeLicenseLimit() {
        ServiceResponse<List<EnterpriseLicenseDTO>> listEnterpriseLicense = this.enterpriseApiService.listEnterpriseLicense(UserDetailUtils.getUserDetail().getEnterpriseId());
        if (listEnterpriseLicense.isSuccess() && CollectionUtils.isNotEmpty(listEnterpriseLicense.getResult())) {
            if (listEnterpriseLicense.getResult().size() == 4) {
                return RestResponse.success(listEnterpriseLicense.getResult().get(3).getUpperLimit());
            }
        }
        return RestResponse.success();
    }

    @RequestMapping("/tran-overflow")
    public RestResponse tranOverFlow(String storeIds) {
        ServiceResponse<Void> serviceResponse = this.storeApiService.tranOverFlow(UserDetailUtils.getUserDetail().getEnterpriseId(), storeIds);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

//    @RequestMapping("list-store-log")
//    public RestResponse listStoreLog(Integer storeId, PageQO pageQO) {
//        Page page = new Page(pageQO.getCurrentPage(), pageQO.getPageSize());
//        Map<String, Object> params = new HashMap<>();
//        params.put("relationId", storeId);
//        page.setParams(params);
//        ServiceResponse response = this.logApiService.listSystemSetLog(page);
//        if (response.isSuccess()) {
//            return RestResponse.success(response.getResult());
//        }
//        return EnterpriseRestResponse.failure(response);
//    }
    @RequestMapping("list-store-log")
    public RestResponse listStoreLog(com.gic.enterprise.qo.PageQO pageQO, String content, Integer reason, Integer logType, Integer storeId) {
        StoreLogDTO storeLog = new StoreLogDTO();
        storeLog.setContent(content);
        storeLog.setReason(reason);
        storeLog.setLogType(logType);
        storeLog.setStoreId(storeId);
        ServiceResponse<Page<StoreLogDTO>> response = storeLogApiService.listStoreLog(storeLog, pageQO.getCurrentPage(), pageQO.getPageSize());
        return ResultControllerUtils.commonResult(response);
    }

    @RequestMapping("bulk-update-store")
    public RestResponse bulkUpdateStatus(String storeIds, String value, int type, Integer toStoreId) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        if (StringUtils.isBlank(storeIds) || StringUtils.isBlank(value)) {
            return EnterpriseRestResponse.failure(ErrorCode.MISS_PARAMETER);
        }
        ServiceResponse response = this.storeApiService.bulkUpdateStore(storeIds, value, enterpriseId, type, UserDetailUtils.getUserDetail().getUserId(), UserDetailUtils.getUserDetail().getUserInfo().getUserName());
        if (response.isSuccess()) {
            if (type == 1 && toStoreId != null) {
                // 如果是批量修改状态 要转移门店
                StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
                storeSearchDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
                storeSearchDTO.setStoreIds(storeIds.replace(',', ' '));
                Page<StoreDTO> page = this.storeApiService.listStore(storeSearchDTO, 1, 1000, "storeInfoId").getResult();
                if (page == null || page.getResult() == null) {
                    return EnterpriseRestResponse.failure(ErrorCode.NOTEXISTS);
                }
                List<String> list = page.getResult().stream().map(e -> e.getStoreInfoId().toString()).collect(Collectors.toList());
                Integer storeInfoId = storeApiService.getStoreById(UserDetailUtils.getUserDetail().getEnterpriseId(), toStoreId).getResult().getStoreInfoId();
                StoreBatchUpdateDTO storeBatchUpdateDTO = new StoreBatchUpdateDTO();
                storeBatchUpdateDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
                storeBatchUpdateDTO.setSourceStoreInfoIdList(list);
                storeBatchUpdateDTO.setTargetStoreInfoId(storeInfoId + "");
                storeBatchUpdateDTO.setUserId(UserDetailUtils.getUserDetail().getUserId());
                logger.info("transfer-store-member:{}", JSON.toJSONString(storeBatchUpdateDTO));
                this.storeUpdateApiService.storeBatchUpdate(storeBatchUpdateDTO);
            }
            // 写日志
            String[] storeArr = storeIds.split(",");
            StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
            storeSearchDTO.setEnterpriseId(enterpriseId);
            storeSearchDTO.setStoreIds(Joiner.on(" ").join(storeArr));
            Page<StoreDTO> page = this.storeApiService.listStore(storeSearchDTO, 1, 3, "storeName").getResult();
            if (page != null && CollectionUtils.isNotEmpty(page.getResult())) {
                List<StoreDTO> storeList = page.getResult();
                if (CollectionUtils.isNotEmpty(storeList)) {
                    String storeName = storeList.stream().map(StoreDTO::getStoreName).reduce((x, y) -> x + "," + y).get();
                    if (storeArr.length > 3) {
                        storeName += "等" + storeArr.length + "家门店";
                    }
                    String content;
                    if (type == 1) {
                        content = "修改启用状态";
                    } else if (type == 7) {
                        content = "批量修改分组";
                    } else {
                        content = "批量编辑";
                    }
                    LogUtils.createLog(content, storeName);
                }
            }
            return RestResponse.success();
        }
        return EnterpriseRestResponse.failure(response);
    }

    @RequestMapping("list-store-export-field")
    public RestResponse listStoreExportField(){
        List<StoreListSourceVO> listSource = this.getStoreAllListSource(UserDetailUtils.getUserDetail().getEnterpriseId());
        return RestResponse.success(EntityUtil.changeEntityListByJSON(StoreExportFieldVO.class, listSource));
    }

    @RequestMapping("export-store-list")
    public RestResponse exportStoreList(HttpServletRequest request, @RequestBody StoreExportQO storeExportQO){
        storeExportQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        ServiceResponse<Long> response = this.storeApiService.queryStoreCountFromES(storeExportQO);
        if(response.isSuccess() && response.getResult() > 0){
            int total = response.getResult().intValue();
            DownloadReport report = new DownloadReport();
            report.setApplyUserId(UserDetailUtils.getUserDetail().getUserId());
            report.setDataContent("门店资料");
            report.setDataCount(total);
            report.setDataType(storeExportQO.getDataType());
            report.setDataUrl(request.getHeader("Referer"));
            report.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
            report.setFileName(storeExportQO.getFileName());
            Integer downloadReport = DataDownloadUtils.createDownloadReport(report);
            String path = RequestContext.getContext().getRequest().getSession().getServletContext().getRealPath("/excel/csv/collaborator/file/");
            List<String> nameList = new ArrayList<>();
            List<StoreListSourceVO> listSource = this.getStoreAllListSource(UserDetailUtils.getUserDetail().getEnterpriseId());
            Map<String, String> map = listSource.stream().collect(Collectors.toMap(e -> e.getValue(), e -> e.getKey()));
            for (String fieldCode: storeExportQO.getFieldCodeList()) {
                nameList.add(map.get(fieldCode));
            }
            List<String> fieldCodeList = storeExportQO.getFieldCodeList();

            ExecutorPoolSingleton.getInstance().executeTask(new Runnable() {
                @Override
                public void run() {
                    DownloadDataLoader<StoreDTO> loader = new DownloadDataLoader<StoreDTO>() {
                        @Override
                        protected List<StoreDTO> getDownloadData(Integer pageNum) throws Exception {
                            ServiceResponse<Page<StoreDTO>> pageServiceResponse = storeApiService.listStore(storeExportQO, pageNum, 1000);
                            List<StoreDTO> list = pageServiceResponse.getResult().getResult();
                            convertCustomField(list);
                            convertStatusName(fieldCodeList, list);
                            return list;
                        }

                        private void convertStatusName(List<String> fieldCodeList, List<StoreDTO> list) {
                            if (CollectionUtils.isNotEmpty(list)) {
                                int completeStatusIndex = fieldCodeList.indexOf("completeStatus");
                                if (completeStatusIndex != -1) {
                                    fieldCodeList.set(completeStatusIndex, "completeStatusName");
                                    for (StoreDTO e : list) {
                                        e.setCompleteStatusName(e.getCompleteStatus() == null ? "未知" : (e.getCompleteStatus() == 1 ? "已完善" : "未完善"));
                                    }
                                }
                                int statusIndex = fieldCodeList.indexOf("status");
                                if (statusIndex != -1) {
                                    fieldCodeList.set(statusIndex, "statusName");
                                    for (StoreDTO e : list) {
                                        e.setStatusName(e.getStatus() == null ? "未知" : (e.getStatus() == 1 ? "已启用" : "未启用"));
                                    }
                                }
                                int erpStatusIndex = fieldCodeList.indexOf("erpStatus");
                                if (erpStatusIndex != -1) {
                                    fieldCodeList.set(erpStatusIndex, "erpStatusName");
                                    Map<Integer, String> erpStatusMap = storeDictApiService.listAllStoreStatus(list.get(0).getEnterpriseId())
                                            .getResult().stream().collect(Collectors.toMap(e -> Integer.valueOf(e.getValue()), StoreDictDTO::getKey));
                                    for (StoreDTO e : list) {
                                        e.setErpStatusName(erpStatusMap.get(e.getErpStatus()));
                                    }
                                }
                                int storeTypeIndex = fieldCodeList.indexOf("storeType");
                                if (storeTypeIndex != -1) {
                                    fieldCodeList.set(storeTypeIndex, "storeTypeName");
                                    Map<Integer, String> storeTypeMap = storeDictApiService.listAllStoreType(list.get(0).getEnterpriseId())
                                            .getResult().stream().collect(Collectors.toMap(e -> Integer.valueOf(e.getValue()), StoreDictDTO::getKey));
                                    for (StoreDTO e : list) {
                                        e.setStoreTypeName(storeTypeMap.get(e.getStoreType()));
                                    }
                                }
                            }
                        }
                    };
                    try {
                        download(path, downloadReport, storeExportQO.getFileName(), storeExportQO.getExcelExtension(), nameList, fieldCodeList, loader, null,  null);
                    } catch (Exception e) {
                        logger.error("异常", e);
                    }
                }
            });
            return RestResponse.success(downloadReport);
        }else {
            return RestResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "没有数据可以导出");
        }
    }

    @RequestMapping("store-qrcode-download")
    public RestResponse storeQrcodeDownload(@RequestBody QrcodeQO qrcodeQO){
        int total = 0;
        if(qrcodeQO.getIsAll() == 1){
            StoreSearchDTO storeSearchDTO = qrcodeQO.getStoreSearchDTO();
            storeSearchDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
            ServiceResponse<Long> serviceResponse = this.storeApiService.queryStoreCountFromES(storeSearchDTO);
            total = serviceResponse.getResult().intValue();
            if(total == 0){
                return RestResponse.failure(ErrorCode.NOTEXISTS.getCode(), "没有门店下载二维码");
            }
        }else {
            total = qrcodeQO.getContent().size();
        }
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        new QrcodeDownload(){
            @Override
            public List<QrcodeContent> getContent(int pageNum, int pageSize){
                if(qrcodeQO.getIsAll() == 1){
                    StoreSearchDTO storeSearchDTO = qrcodeQO.getStoreSearchDTO();
                    storeSearchDTO.setEnterpriseId(enterpriseId);
                    ServiceResponse<Page<StoreDTO>> response = storeApiService.listStore(storeSearchDTO, pageNum, pageNum);
                    if(response.isSuccess() && response.getResult() != null){
                        Page<StoreDTO> result = response.getResult();
                        if(CollectionUtils.isNotEmpty(result.getResult())){
                            List<QrcodeContent> list = new ArrayList<>();
                            for(StoreDTO dto : result.getResult()){
                                QrcodeContent qrcodeContent = new QrcodeContent();
                                qrcodeContent.setCustomParams(dto.getStoreId()+"");
                                qrcodeContent.setTitle(dto.getStoreName());
                                list.add(qrcodeContent);
                            }
                            return list;
                        }
                    }
                }else {
                    return EntityUtil.changeEntityListByJSON(QrcodeContent.class, qrcodeQO.getContent());
                }
                return null;
            }
        }.takeQrcode(EntityUtil.changeEntityByJSON(com.gic.download.qo.QrcodeQO.class, qrcodeQO), total, QrcodeTypeEnum.STORE.getType(),
                downloadReportApiService, weixinBaseFunService, weixinWxaFunService, qrCodeApiService);
        return RestResponse.success();
    }

    @RequestMapping("subject-cost-store-search")
    public RestResponse subjectCostStoreSearch(String search){
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        storeSearchDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        storeSearchDTO.setStatus(StoreStatusEnum.ONLINE.getCode());
        storeSearchDTO.setSearch(search);
        ServiceResponse<Page<StoreDTO>> response = this.storeApiService.listStore(storeSearchDTO, 1, 1000);
        if(response.isSuccess() && response.getResult() != null){
            List<StoreDTO> result = response.getResult().getResult();
            if(CollectionUtils.isEmpty(result)){
                result = new ArrayList<StoreDTO>();
            }
            if("总部".equals(search)){
                StoreDTO storeDTO = new StoreDTO();
                storeDTO.setStoreName("总部");
                storeDTO.setStoreId(8888);
                result.add(storeDTO);
            }
            if("无归属".equals(search)){
                StoreDTO storeDTO = new StoreDTO();
                storeDTO.setStoreName("无归属");
                storeDTO.setStoreId(9999);
                result.add(storeDTO);
            }
            return RestResponse.success(result);
        }
        return RestResponse.success();
    }

    @RequestMapping("has-store-member")
    public RestResponse hasStoreMember(Integer storeId) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Integer storeInfoId = storeApiService.getStoreById(enterpriseId, storeId).getResult().getStoreInfoId();
        ServiceResponse<Boolean> serviceResponse = storeUpdateApiService.hasMemberByMainStore(enterpriseId, storeInfoId);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("transfer-store-member")
    public RestResponse transferStoreMember(String storeId, Integer toStoreId){
        // 把参数改成storeInfoId
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        storeSearchDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        List<StoreDTO> storeInfoList;
        if (StringUtils.equals(storeId, "all")) {
            storeInfoList = this.storeApiService.listStore(storeSearchDTO, 1, 10000, "storeInfoId").getResult().getResult();

        } else {
            storeSearchDTO.setStoreIds(storeId.replace(',', ' '));
            storeInfoList = this.storeApiService.listStore(storeSearchDTO, 1, storeId.split(" ").length, "storeInfoId").getResult().getResult();
        }
        List<String> list = storeInfoList.stream().map(e -> e.getStoreInfoId().toString()).collect(Collectors.toList());
        Integer storeInfoId = storeApiService.getStoreById(UserDetailUtils.getUserDetail().getEnterpriseId(), toStoreId).getResult().getStoreInfoId();

//        List<String> list = Arrays.asList(storeId.split(","));
        StoreBatchUpdateDTO storeBatchUpdateDTO = new StoreBatchUpdateDTO();
        storeBatchUpdateDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        storeBatchUpdateDTO.setSourceStoreInfoIdList(list);
        storeBatchUpdateDTO.setTargetStoreInfoId(storeInfoId+"");
        storeBatchUpdateDTO.setUserId(UserDetailUtils.getUserDetail().getUserId());
        logger.info("transfer-store-member:{}", JSON.toJSONString(storeBatchUpdateDTO));
        ServiceResponse<Void> response = this.storeUpdateApiService.storeBatchUpdate(storeBatchUpdateDTO);
        logger.info("transfer-store-member-result:{}", JSON.toJSONString(response));
        for (String store : list) {
            StoreDTO storeDTO = storeApiService.getStoreById(UserDetailUtils.getUserDetail().getEnterpriseId(), Integer.valueOf(store)).getResult();
            if (storeDTO != null) {
                LogUtils.createLog("会员处理", storeDTO.getStoreName());
            }
        }
        return EnterpriseRestResponse.failure(response);
    }

    private List<StoreListSourceVO> getStoreAllListSource(Integer enterpriseId) {
        List<StoreListSourceVO> list = new ArrayList<>();
        ServiceResponse<List<StoreDictDTO>> serviceResponse = this.storeDictApiService.listStoreField(enterpriseId);
        if (serviceResponse.isSuccess()) {
            List<StoreDictDTO> storeDictDTOList = serviceResponse.getResult();
            if (CollectionUtils.isNotEmpty(storeDictDTOList)) {
                for (StoreDictDTO storeDictDTO : storeDictDTOList) {
                    if (storeDictDTO.getChecked() == 1) {
                        StoreListSourceVO vo = new StoreListSourceVO();
                        vo.setChecked(0);
                        vo.setIsCustom(1);
                        vo.setKey(storeDictDTO.getKey());
                        String key = StoreRedisKeyUtils.getStoreFieldKey(enterpriseId, Integer.valueOf(storeDictDTO.getValue()));
                        Object object = RedisUtil.getCache(key);
                        if(object != null){
                            vo.setValue(object.toString());
                            list.add(vo);
                        }
                    }
                }
            }
        }
        list.add(new StoreListSourceVO(StoreESFieldsEnum.STORENAME.getField(), StoreESFieldsEnum.STORENAME.getDesc(), 1));
        list.add(new StoreListSourceVO(StoreESFieldsEnum.STOREGROUPNAME.getField(), StoreESFieldsEnum.STOREGROUPNAME.getDesc(), 1));
        list.add(new StoreListSourceVO(StoreESFieldsEnum.STORECODE.getField(), StoreESFieldsEnum.STORECODE.getDesc(), 1));
        list.add(new StoreListSourceVO(StoreESFieldsEnum.PROVINCES.getField(), StoreESFieldsEnum.PROVINCES.getDesc()));
        list.add(new StoreListSourceVO(StoreESFieldsEnum.STOREBRANDS.getField(), StoreESFieldsEnum.STOREBRANDS.getDesc() ));
        list.add(new StoreListSourceVO(StoreESFieldsEnum.COMPLETESTATUS.getField(), StoreESFieldsEnum.COMPLETESTATUS.getDesc()));
        list.add(new StoreListSourceVO(StoreESFieldsEnum.STATUS.getField(), StoreESFieldsEnum.STATUS.getDesc()));
        list.add(new StoreListSourceVO(StoreESFieldsEnum.REGIONNAME.getField(), StoreESFieldsEnum.REGIONNAME.getDesc()));
        list.add(new StoreListSourceVO(StoreESFieldsEnum.ERPSTATUS.getField(), StoreESFieldsEnum.ERPSTATUS.getDesc()));
        list.add(new StoreListSourceVO(StoreESFieldsEnum.STORETYPE.getField(), StoreESFieldsEnum.STORETYPE.getDesc()));
        list.add(new StoreListSourceVO(StoreESFieldsEnum.STOREID.getField(), StoreESFieldsEnum.STOREID.getDesc(),1));
        return list;
    }

    private Integer getIsEditGroup(Integer enterpriseId, StoreDTO storeDTO) {
        if (storeDTO.getIsEditStoreGroup() != null && storeDTO.getIsEditStoreGroup() == 1) {
            return 1;
        }
        Integer isEditGroup;
        Integer newGroupId = storeDTO.getStoreGroupId();
        if (storeDTO.getStoreId() == null) {
            isEditGroup = newGroupId == null ? 0 : 1;
        } else {
            StoreDTO oldDTO = storeApiService.getStoreById(enterpriseId, storeDTO.getStoreId()).getResult();
            if (oldDTO == null) {
                return 0;
            }
            Integer oldGroupId = oldDTO.getStoreGroupId();
            if (newGroupId == null && oldGroupId == null) {
                isEditGroup = 0;
            } else if (oldGroupId != null) {
                isEditGroup = oldGroupId.equals(newGroupId) ? 0 : 1;
            } else {
                isEditGroup = newGroupId.equals(oldGroupId) ? 0 : 1;
            }
        }
        return isEditGroup;
    }
    private void convertCustomField(List<StoreDTO> list){
        try{
            Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
            for(StoreDTO dto : list){
                if(dto.getC1() != null){
                    String value = this.convert(enterpriseId, "c1", dto.getC1());
                    if(StringUtils.isNotBlank(value)){
                        dto.setC1(value);
                    }
                }
                if(dto.getC2() != null){
                    String value = this.convert(enterpriseId, "c2", dto.getC2());
                    if(StringUtils.isNotBlank(value)){
                        dto.setC2(value);
                    }
                }
                if(dto.getC3() != null){
                    String value = this.convert(enterpriseId, "c3", dto.getC3());
                    if(StringUtils.isNotBlank(value)){
                        dto.setC3(value);
                    }
                }
                if(dto.getC4() != null){
                    String value = this.convert(enterpriseId, "c4", dto.getC4());
                    if(StringUtils.isNotBlank(value)){
                        dto.setC4(value);
                    }
                }
                if(dto.getC5() != null){
                    String value = this.convert(enterpriseId, "c5", dto.getC5());
                    if(StringUtils.isNotBlank(value)){
                        dto.setC5(value);
                    }
                }
                if(dto.getC6() != null){
                    String value = this.convert(enterpriseId, "c6", dto.getC6());
                    if(StringUtils.isNotBlank(value)){
                        dto.setC6(value);
                    }
                }
                if(dto.getC7() != null){
                    String value = this.convert(enterpriseId, "c7", dto.getC7());
                    if(StringUtils.isNotBlank(value)){
                        dto.setC7(value);
                    }
                }
                if(dto.getC8() != null){
                    String value = this.convert(enterpriseId, "c8", dto.getC8());
                    if(StringUtils.isNotBlank(value)){
                        dto.setC8(value);
                    }
                }
                if(dto.getC9() != null){
                    String value = this.convert(enterpriseId, "c9", dto.getC9());
                    if(StringUtils.isNotBlank(value)){
                        dto.setC9(value);
                    }
                }
                if(dto.getC10() != null){
                    String value = this.convert(enterpriseId, "c10", dto.getC10());
                    if(StringUtils.isNotBlank(value)){
                        dto.setC10(value);
                    }
                }
            }
        }catch (Exception e){
            logger.warn("转换异常", e);
        }
    }

    private String convert(Integer enterpriseId, String cloumn, String value) throws ParseException {
        String keyc = StoreRedisKeyUtils.getStoreFieldKeyC(enterpriseId, cloumn);
        Object cache = RedisUtil.getCache(keyc);
        if(cache != null){
            ServiceResponse<StoreFieldDTO> storeField = this.storeFieldApiService.getStoreField(Integer.valueOf(cache.toString()));
            if(storeField.isSuccess() && storeField.getResult() != null){
                if(storeField.getResult().getStoreFieldType() == StoreFieldTypeEnum.DATE.getCode()){
                    JSONObject jsonObject = JSON.parseObject(storeField.getResult().getStoreFieldDetail());
                    return this.getDateValue(jsonObject.getIntValue("dateType"), value);
                }
                if(storeField.getResult().getStoreFieldType() == StoreFieldTypeEnum.RADIO.getCode() ||
                        storeField.getResult().getStoreFieldType() == StoreFieldTypeEnum.CHECK.getCode()){
                    String[] arr = value.split(",");
                    String valueStr = "";
                    for(String s : arr){
                        ServiceResponse<StoreFieldSelectDTO> storeFieldSelect = this.storeFieldSelectApiService.getStoreFieldSelect(Integer.valueOf(s));
                        if(storeFieldSelect.isSuccess() && storeFieldSelect.getResult()!= null){
                            valueStr += storeFieldSelect.getResult().getStoreFieldSelectName()+",";
                        }
                    }
                    if(StringUtils.isNotBlank(valueStr)){
                        return valueStr.substring(0, valueStr.length()-1);
                    }
                }
            }
        }
        return null;
    }

    private String getDateValue(int dateType, String value) {
        try{
            return this.getToDateStrFormat(dateType).format(this.getToDateFormat(dateType).parse(value));
        }catch (Exception e){
            logger.warn(e);
        }
        return null;
    }

    private SimpleDateFormat getToDateStrFormat(int dateType){
        SimpleDateFormat simpleDateFormat = map.get(dateType);
        if(simpleDateFormat == null){
            if(StoreFieldDateEnum.YMDHMS.getCode() == dateType){
                simpleDateFormat = new SimpleDateFormat("YYYY-MM-dd HH:mm:ss");
            }
            if(StoreFieldDateEnum.YMD.getCode() == dateType){
                simpleDateFormat = new SimpleDateFormat("YYYY-MM-dd");
            }
            if(StoreFieldDateEnum.YM.getCode() == dateType){
                simpleDateFormat = new SimpleDateFormat("YYYY-MM");
            }
            if(StoreFieldDateEnum.MD.getCode() == dateType){
                simpleDateFormat = new SimpleDateFormat("MM-dd");
            }
            if(StoreFieldDateEnum.Hms.getCode() == dateType){
                simpleDateFormat = new SimpleDateFormat("YYYY-MM-dd HH:mm:ss");
            }
            if(simpleDateFormat == null){
                simpleDateFormat = new SimpleDateFormat("YYYY-MM-dd");
            }
            map.put(dateType, simpleDateFormat);
        }
        return simpleDateFormat;
    }

    private SimpleDateFormat getToDateFormat(int dateType){
        SimpleDateFormat simpleDateFormat = map1.get(dateType);
        if(simpleDateFormat == null){
            if(StoreFieldDateEnum.YMDHMS.getCode() == dateType){
                simpleDateFormat = new SimpleDateFormat("YYYYMMddHHmmss");
            }
            if(StoreFieldDateEnum.YMD.getCode() == dateType){
                simpleDateFormat = new SimpleDateFormat("YYYYMMdd");
            }
            if(StoreFieldDateEnum.YM.getCode() == dateType){
                simpleDateFormat = new SimpleDateFormat("YYYYMM");
            }
            if(StoreFieldDateEnum.MD.getCode() == dateType){
                simpleDateFormat = new SimpleDateFormat("MMdd");
            }
            if(StoreFieldDateEnum.Hms.getCode() == dateType){
                simpleDateFormat = new SimpleDateFormat("YYYYMMddHHmmss");
            }
            if(simpleDateFormat == null){
                simpleDateFormat = new SimpleDateFormat("YYYYMMdd");
            }
            map1.put(dateType, simpleDateFormat);
        }
        return simpleDateFormat;
    }
}